# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Tests for bayesian_structural_analysis

Author : Bertrand Thirion, 2009
"""
#autoindent

import numpy as np
import scipy.stats as st
from nose.tools import assert_true

from nipy.testing import dec

from ...utils.simul_multisubject_fmri_dataset import surrogate_2d_dataset
from ..bayesian_structural_analysis import compute_BSA_simple
from ..discrete_domain import domain_from_binary_array


def make_bsa_2d(betas, theta=3., dmax=5., ths=0, thq=0.5, smin=0,
                        nbeta=[0], method='simple'):
    """
    Function for performing bayesian structural analysis on a set of images.

    Fixme: 'quick' is not tested
    """
    ref_dim = np.shape(betas[0])
    n_subj = betas.shape[0]
    xyz = np.array(np.where(betas[:1])).T
    nvox = np.size(xyz, 0)

    # get the functional information
    lbeta = np.array([np.ravel(betas[k]) for k in range(n_subj)]).T

    # the voxel volume is 1.0
    g0 = 1.0 / (1.0 * nvox)
    bdensity = 1
    dom = domain_from_binary_array(np.ones(ref_dim))

    if method == 'simple':
        group_map, AF, BF, likelihood = \
                   compute_BSA_simple(dom, lbeta, dmax, thq, smin, ths,
                                      theta, g0, bdensity)
    return AF, BF


@dec.slow
def test_bsa_methods():
    # generate the data
    n_subj = 5
    shape = (40, 40)
    pos = np.array([[12, 14],
                    [20, 20],
                    [30, 35]])
    # make a dataset with a nothing feature
    null_ampli = np.array([0, 0, 0])
    null_betas = surrogate_2d_dataset(n_subj=n_subj,
                                        shape=shape,
                                        pos=pos,
                                        ampli=null_ampli,
                                        width=5.0,
                                        seed=1)
    #null_betas = np.reshape(null_dataset, (n_subj, shape[0], shape[1]))
    # make a dataset with a something feature
    pos_ampli = np.array([5, 7, 6])
    pos_betas = surrogate_2d_dataset(n_subj=n_subj,
                                       shape=shape,
                                       pos=pos,
                                       ampli=pos_ampli,
                                       width=5.0,
                                       seed=2)
    #pos_betas = np.reshape(pos_dataset, (n_subj, shape[0], shape[1]))
    # set various parameters
    theta = float(st.t.isf(0.01, 100))
    dmax = 5. / 1.5
    half_subjs = n_subj / 2
    thq = 0.9
    smin = 5

    # tuple of tuples with each tuple being
    # (name_of_method, ths_value, data_set, test_function)
    algs_tests = (
        ('simple', half_subjs, null_betas, lambda AF, BF: AF.k == 0),
        ('simple', 1, pos_betas, lambda AF, BF: AF.k > 1))

    for name, ths, betas, test_func in algs_tests:
        # run the algo
        AF, BF = make_bsa_2d(betas, theta, dmax, ths, thq, smin, method=name)
        yield assert_true, test_func(AF, BF)


if __name__ == '__main__':
    import nose
    nose.run(argv=['', __file__])



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
'''
This module contains the one-parameter exponential families used
for fitting GLMs and GAMs.

These families are described in

   P. McCullagh and J. A. Nelder.  "Generalized linear models."
   Monographs on Statistics and Applied Probability.
   Chapman & Hall, London, 1983.

'''

from .family import (Gaussian, Family, Poisson, Gamma, InverseGaussian,
                     Binomial)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
#autoindent

"""
Generic Parcellation class:
Contains all the items that define a multi-subject parcellation

Author : Bertrand Thirion, 2005-2008

TODO : add a method 'global field', i.e. non-subject-specific info
"""

import numpy as np

###################################################################
# Parcellation class
###################################################################


class MultiSubjectParcellation(object):
    """
    MultiSubjectParcellation class are used to represent parcels
    that can have different spatial different contours
    in a given group of subject
    It consists of
    self.domain: the specification of a domain
    self.template_labels the specification of a template parcellation
    self.individual_labels the specification of individual parcellations

    fixme:should inherit from mroi.MultiROI
    """

    def __init__(self, domain, template_labels=None, individual_labels=None,
                 nb_parcel=None):
        """ Initialize multi-subject parcellation

        Parameters
        ----------
        domain: discrete_domain.DiscreteDomain instance,
                definition of the space considered in the parcellation
        template_labels: array of shape domain.size, optional
                         definition of the template labelling
        individual_labels: array of shape (domain.size, nb_subjects), optional,
                           the individual parcellations
                           corresponding to the template
        nb_parcel: int, optional,
                   number of parcels in the model
                   can be inferred as template_labels.max()+1, or 1 by default
                   cannot be smaller than template_labels.max()+1
        """
        self.domain = domain
        self.template_labels = template_labels
        self.individual_labels = individual_labels
        self.nb_parcel = 1
        if template_labels is not None:
            self.nb_parcel = template_labels.max() + 1
        if nb_parcel is not None:
            self.nb_parcel = nb_parcel

        self.check()
        self.nb_subj = 0
        if individual_labels is not None:
            if individual_labels.shape[0] == individual_labels.size:
                self.individual_labels = individual_labels[:, np.newaxis]
            self.nb_subj = self.individual_labels.shape[1]

        self.features = {}

    def copy(self):
        """ Returns a copy of self
        """
        msp = MultiSubjectParcellation(self.domain.copy(),
                                        self.template_labels.copy(),
                                        self.individual_labels.copy(),
                                        self.nb_parcel)
        for fid in self.features.keys():
            msp.set_feature(fid, self.get_feature(fid).copy())
        return msp

    def check(self):
        """ Performs an elementary check on self
        """
        size = self.domain.size
        if self.template_labels is not None:
            nvox = np.size(self.template_labels)
            if size != nvox:
                raise ValueError("template labels not consistent with domain")
        if self.individual_labels is not None:
            n2 = self.individual_labels.shape[0]
            if size != n2:
                raise ValueError(
                    "Individual labels not consistent with domain")
        if self.nb_parcel < self.template_labels.max() + 1:
            raise ValueError("too many labels in template")
        if self.nb_parcel < self.individual_labels.max() + 1:
            raise ValueError("Too many labels in individual models")

    def set_template_labels(self, template_labels):
        """
        """
        self.template_labels = template_labels
        self.check()

    def set_individual_labels(self, individual_labels):
        """
        """
        self.individual_labels = individual_labels
        self.check()
        self.nb_subj = self.individual_labels.shape[1]

    def population(self):
        """ Returns the counting of labels per voxel per subject

        Returns
        -------
        population: array of shape (self.nb_parcel, self.nb_subj)
        """
        population = np.zeros((self.nb_parcel, self.nb_subj)).astype(np.int)
        for ns in range(self.nb_subj):
            for k in range(self.nb_parcel):
                population[k, ns] = np.sum(self.individual_labels[:, ns] == k)
        return population

    def make_feature(self, fid, data):
        """ Compute parcel-level averages of data

        Parameters
        ----------
        fid: string, the feature identifier
        data: array of shape (self.domain.size, self.nb_subj, dim) or
              (self.domain.sire, self.nb_subj)
              Some information at the voxel level

        Returns
        -------
        pfeature: array of shape(self.nb_parcel, self.nbsubj, dim)
                  the computed feature data
        """
        if len(data.shape) < 2:
            raise ValueError("Data array should at least have dimension 2")
        if len(data.shape) > 3:
            raise ValueError("Data array should have <4 dimensions")
        if ((data.shape[0] != self.domain.size) or
            (data.shape[1] != self.nb_subj)):
            raise ValueError('incorrect feature size')

        if len(data.shape) == 3:
            dim = data.shape[2]
            pfeature = np.zeros((self.nb_parcel, self.nb_subj, dim))
        else:
            pfeature = np.zeros((self.nb_parcel, self.nb_subj))

        for k in range(self.nb_parcel):
            for s in range(self.nb_subj):
                dsk = data[self.individual_labels[:, s] == k, s]
                pfeature[k, s] = np.mean(dsk, 0)

        self.set_feature(fid, pfeature)
        return pfeature

    def set_feature(self, fid, data):
        """ Set feature defined by `fid` and `data` into ``self``

        Parameters
        ----------
        fid: string
            the feature identifier
        data: array of shape (self.nb_parcel, self.nb_subj, dim) or
              (self.nb_parcel, self.nb_subj)
            the data to be set as parcel- and subject-level information
        """
        if len(data.shape) < 2:
            raise ValueError("Data array should at least have dimension 2")
        if (data.shape[0] != self.nb_parcel) or \
                (data.shape[1] != self.nb_subj):
            raise ValueError('incorrect feature size')
        else:
            self.features.update({fid: data})

    def get_feature(self, fid):
        """ Get feature defined by `fid`

        Parameters
        ----------
        fid: string, the feature identifier
        """
        return self.features[fid]



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np
from scipy.ndimage import correlate1d, _ni_support, gaussian_filter, \
                binary_erosion
from scipy import math

def square_gaussian_filter1d(input, sigma, axis = -1, output = None, mode = "reflect", cval = 0.0):
    """One-dimensional Squared Gaussian filter.

    The standard-deviation of the Gaussian filter is given by
    sigma.
    """
    sd = float(sigma)
    # make the length of the filter equal to 4 times the standard
    # deviations:
    lw = int(4.0 * sd + 0.5)
    weights = [0.0] * (2 * lw + 1)
    weights[lw] = 1.0
    sum = 1.0
    sd = sd * sd
    # calculate the kernel:
    for ii in range(1, lw + 1):
        tmp = math.exp(- 0.5 * float(ii * ii) / sd)
        weights[lw + ii] = tmp
        weights[lw - ii] = tmp
        sum += 2.0 * tmp
    for ii in range(2 * lw + 1):
        weights[ii] /= sum
    weights[ii] = weights[ii]**2
    return correlate1d(input, weights, axis, output, mode, cval, 0)


def square_gaussian_filter(input, sigma, output = None, mode = "reflect", cval = 0.0):
    """Multi-dimensional Squared Gaussian filter.

    The standard-deviations of the Gaussian filter are given for each
    axis as a sequence, or as a single number, in which case it is
    equal for all axes.

    Note: The multi-dimensional filter is implemented as a sequence of
    one-dimensional convolution filters. The intermediate arrays are
    stored in the same data type as the output. Therefore, for output
    types with a limited precision, the results may be imprecise
    because intermediate results may be stored with insufficient
    precision.
    """
    input = np.asarray(input)
    output, return_value =_ni_support._get_output(output, input)
    sigmas =_ni_support._normalize_sequence(sigma, input.ndim)
    axes = range(input.ndim)
    axes = [(axes[ii], sigmas[ii])
                        for ii in range(len(axes)) if sigmas[ii] > 1e-15]
    if len(axes) > 0:
        for axis, sigma in axes:
            square_gaussian_filter1d(input, sigma, axis, output,
                              mode, cval)
            input = output
    else:
        output[...] = input[...]
    return return_value


class displacement_field(object):
    """
    Sampling of multiple vector-valued displacement fields on a 3D-lattice.
    Displacement fields are generated as linear combinations of fixed displacements.
    The coefficients are random Gaussian variables.
    """
    def __init__(self, XYZ, sigma, n=1, mask=None, step=None):
        """
        Input :
        XYZ          (3,p)   array of voxel coordinates
        sigma        <float> standard deviate of Gaussian filter kernel
                             Each displacement block has length 4*sigma
        n            <int>   number of generated displacement fields.
        mask         (q,)    displacement blocks are limited to mask
        The constructor creates the following fields :
        self.block           List of N block masks (voxel index vectors)
        self.weights         List of N block weights (same shape as the masks)
        self.U       (3,n,N) Displacement coefficients
        self.V       (3,n,p) Displacements
        self.W       (3,n,p) Discretize displacements
        self.I       (n,p)   Displaced voxels index
                             (voxel k in the mask is displaced by field i to voxel self.I[i,k])
        """
        self.XYZ = XYZ
        self.sigma = sigma
        if np.isscalar(sigma):
            self.sigma = sigma * np.ones(3)
        self.n = n
        self.XYZ_min = self.XYZ.min(axis=1).reshape(3, 1) - 1
        self.XYZ_max = self.XYZ.max(axis=1).reshape(3, 1) + 1
        p = XYZ.shape[1]
        if mask == None:
            self.mask = np.arange(p)
        else:
            self.mask = mask
        if step == None:
            self.step = int(round(2 * self.sigma.max()))
        else:
            self.step = step
        self.V = np.zeros((3, n, p), float)
        self.W = np.zeros((3, n, p), int)
        self.I = np.arange(p) * np.ones((n, p), int)
        self.XYZ_vol = np.zeros(XYZ.max(axis=1) + 2, int) - 1
        self.XYZ_vol[XYZ[0], XYZ[1], XYZ[2]] = np.arange(p)
        self.init_displacement_blocks()
        self.compute_inner_blocks()
        self.U = np.zeros((3, n, len(self.block)), float)

    def init_displacement_blocks(self):
        """
        Called by class constructor
        """
        XYZ = self.XYZ
        # displacement kernel
        sigma = self.sigma.max()
        #r = int(round(2 * sigma))
        d = int(round(6 * sigma))
        block_dim = (\
            self.XYZ.max(axis=1)+1 - \
            self.XYZ.min(axis=1)).clip(1,d)
        #kernel = np.zeros(d * np.ones(3), float)
        kernel = np.zeros(block_dim, float)
        kernel[block_dim[0]/2-1:block_dim[0]/2+1,
        block_dim[1]/2-1:block_dim[1]/2+1,
        block_dim[2]/2-1:block_dim[2]/2+1] += 1
        kernel = gaussian_filter(kernel.squeeze(), sigma, mode='constant')
        kernel = kernel.reshape(block_dim)
        kernel /= kernel.max()
        # displacement 'blocks'
        self.block = []
        self.weights = []
        mask_vol = np.zeros(XYZ.max(axis=1) + 2, int) - 1
        mask_vol[list(XYZ[:, self.mask])] = self.mask
        Xm, Ym, Zm = XYZ.min(axis=1).astype(int)
        XM, YM, ZM = XYZ.max(axis=1).clip(1,np.inf).astype(int)
        for i in xrange(Xm, XM, self.step):
            for j in xrange(Ym, YM, self.step):
                for k in xrange(Zm, ZM, self.step):
                    block_vol = mask_vol[i:i + d, j:j + d, k:k + d]
                    XYZ_block = np.array( np.where( block_vol > -1 ) )
                    if XYZ_block.size > 0 \
                    and (kernel[list(XYZ_block)] > 0.05).sum() == (kernel > 0.05).sum():
                        #print i,j,k
                        self.block.append(block_vol[XYZ_block[0], XYZ_block[1], XYZ_block[2]])
                        self.weights.append(kernel[XYZ_block[0], XYZ_block[1], XYZ_block[2]])

    def compute_inner_blocks(self):
        """
        Generate self.inner_blocks, index of blocks which are "far from" the borders of the lattice.
        """
        XYZ = self.XYZ
        sigma = self.sigma.max()
        mask_vol = np.zeros(XYZ.max(axis=1) + 1, int)
        mask_vol[XYZ[0], XYZ[1], XYZ[2]] += 1
        mask_vol = binary_erosion(mask_vol.squeeze(), iterations=int(round(sigma))).astype(int)
        mask_vol = mask_vol.reshape(XYZ.max(axis=1) + 1).astype(int)
        inner_mask = mask_vol[XYZ[0], XYZ[1], XYZ[2]]
        inner_blocks = []
        for i in xrange(len(self.block)):
            if inner_mask[self.block[i]].min() == 1:
                inner_blocks.append(i)
        self.inner_blocks = np.array(inner_blocks)

    def sample(self, i, b, proposal='prior', proposal_std=None, proposal_mean=None):
        """
        Generates U, V, L, W, I, where U, V, W, I are proposals for
        self.U[:,i,b], self.V[:,i,block], self.W[:,i,L], self.I[i,L] if block = self.block[b].
        W and I are given only in those voxels, indexed by L, where they differ from current values.
        Proposal is either 'prior', 'rand_walk' or 'fixed'
        """
        block = self.block[b]
        # Current values
        Uc = self.U[:, i, b]
        Vc = self.V[:, i, block]
        Wc = self.W[:, i, block]
        Ic = self.I[i, block]
        # Proposals
        valid_proposal = False
        while not valid_proposal:
            if proposal == 'prior':
                U = np.random.randn(3) * proposal_std
            elif proposal == 'rand_walk':
                U = Uc + np.random.randn(3) * proposal_std
            else:
                U = proposal_mean + np.random.randn(3) * proposal_std
            V = Vc + (self.weights[b].reshape(1, -1) * (U - Uc).reshape(3,1))
            #print U
            W = np.round(V).astype(int)
            L = np.where((W == Wc).prod(axis=0) == 0)[0]
            XYZ_W = np.clip(self.XYZ[:, block[L]] + W[:, L], self.XYZ_min, self.XYZ_max)
            I = self.XYZ_vol[XYZ_W[0], XYZ_W[1], XYZ_W[2]]
            #print (I == -1).sum()
            if len(L) == 0:
                valid_proposal = True
            elif min(I) > -1:
                valid_proposal = True
        return U, V, block[L], W[:, L], I

    def sample_all_blocks(self, proposal_std=None, proposal_mean=None):
        """
        Generates U, V, W, I, proposals for self.U[:, i], self.V[:, i], self.W[:, i], self.I[i].
        Proposal is either 'prior', 'rand_walk' or 'fixed'
        """
        B = len(self.block)
        p = self.XYZ.shape[1]
        V = np.zeros((3, p), float)
        I = -np.ones(p, int)
        while min(I) == -1:
            U = np.random.randn(3, B) * proposal_std
            if proposal_mean != None:
                U += proposal_mean
            V *= 0
            for b in xrange(B):
                V[:, self.block[b]] += self.weights[b].reshape(1, -1) * U[:, b].reshape(3,1)
            W = np.round(V).astype(int)
            XYZ_W = np.clip(self.XYZ + W, self.XYZ_min, self.XYZ_max)
            I = self.XYZ_vol[XYZ_W[0], XYZ_W[1], XYZ_W[2]]
        return U, V, W, I


class gaussian_random_field(object):
    def __init__(self, XYZ, sigma, n=1):
        self.XYZ = XYZ
        self.sigma = sigma
        if np.isscalar(sigma):
            self.sigma = sigma * (XYZ.max(axis=1) > 1)
        self.n = n
        self.XYZ_vol = np.zeros(XYZ.max(axis=1) + 2, int) - 1
        p = XYZ.shape[1]
        self.XYZ_vol[list(XYZ)] = np.arange(p)
        mask_vol = np.zeros(XYZ.max(axis=1) + 1, int)
        mask_vol[list(XYZ)] += 1
        mask_vol = binary_erosion(mask_vol.squeeze(), iterations=int(round(1.5*self.sigma.max())))
        mask_vol = mask_vol.reshape(XYZ.max(axis=1) + 1).astype(int)
        XYZ_mask = np.array(np.where(mask_vol > 0))
        self.mask = self.XYZ_vol[XYZ_mask[0], XYZ_mask[1], XYZ_mask[2]]
        q = len(self.mask)
        dX, dY, dZ = XYZ.max(axis=1) + 1
        self.U_vol = np.zeros((3, dX, dY, dZ), float)
        self.U_vol[:, XYZ_mask[0], XYZ_mask[1], XYZ_mask[2]] += 1
        self.U_vol = square_gaussian_filter(self.U_vol, [0, self.sigma[0], self.sigma[1], self.sigma[2]], mode='constant')
        self.norm_coeff = 1 / np.sqrt(self.U_vol.max())
        self.U = np.zeros((3, n, q), float)
        self.V = np.zeros((3, n, p), float)
        self.W = np.zeros((3, n, p), int)
        self.I = np.arange(p).reshape(1, p) * np.ones((n, 1), int)
        self.XYZ_min = self.XYZ.min(axis=1).reshape(3, 1) - 1
        self.XYZ_max = self.XYZ.max(axis=1).reshape(3, 1) + 1

    def sample(self, i, std):
        mask = self.mask
        q = len(mask)
        XYZ = self.XYZ
        sigma = self.sigma
        Wc = self.W[:, i]
        valid = False
        if np.isscalar(std):
            std = std * np.ones((3,1))
        while not valid:
            U = np.random.randn(3, q) * std
            self.U_vol *= 0
            self.U_vol[:, XYZ[0, mask], XYZ[1, mask], XYZ[2, mask]] = U
            self.U_vol = gaussian_filter(self.U_vol, [0, sigma[0], sigma[1], sigma[2]], mode='constant')
            V = self.U_vol[:, XYZ[0], XYZ[1], XYZ[2]] * self.norm_coeff
            W = np.round(V).astype(int)
            L = np.where((W == Wc).prod(axis=0) == 0)[0]
            XYZ_W = np.clip(XYZ[:, L] + W[:, L], self.XYZ_min, self.XYZ_max)
            I = self.XYZ_vol[XYZ_W[0], XYZ_W[1], XYZ_W[2]]
            if len(L) == 0:
                valid = True
            elif min(I) > -1:
                valid = True
        #self.U[:, i], self.V[:, i], self.W[:, i, L], self.I[i, L] = U, V, W[:, L], I
        return U, V, L, W[:, L], I



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" Example script for group permutation testing """
from __future__ import print_function # Python 2/3 compatibility

import numpy as np

from nipy.labs.group import permutation_test as PT


def make_data(n=10, mask_shape=(10, 10, 10), axis=0, r=3, signal=5):
    """
    Generate Gaussian noise in a cubic volume
    + cubic activations
    """
    mask = np.zeros(mask_shape, int)
    XYZ = np.array(np.where(mask==0))
    p = XYZ.shape[1]
    data = np.random.randn(n, p)
    I = np.where(np.square(XYZ - XYZ.max(axis=1).reshape(-1, 1) / 2).sum(
            axis=0) <= r ** 2)[0]
    data[:, I] += signal
    vardata = np.random.randn(n, p) ** 2
    if axis == 1:
        data = data.T
        vardata = vardata.T
    return data, vardata, XYZ

###############################################################################
# Example for using permutation_test_onesample class
data, vardata, XYZ = make_data()

# rfx calibration
P = PT.permutation_test_onesample(data, XYZ)

# clusters definition (height threshold, max diameter)
c = [(P.random_Tvalues[P.ndraws * (0.95)], None)]

# regions definition (label vector)
r = np.ones(data.shape[1], int)
r[data.shape[1]/2.:] *= 10
voxel_results, cluster_results, region_results = \
                P.calibrate(nperms=100, clusters=c, regions=[r])

# mfx calibration
P = PT.permutation_test_onesample(data, XYZ, vardata=vardata,
                                  stat_id="student_mfx")
voxel_results, cluster_results, region_results = \
                P.calibrate(nperms=100, clusters=c, regions=[r])

###############################################################################
# Example for using permutation_test_twosample class
data, vardata, XYZ = make_data(n=20)
data1, vardata1, data2, vardata2 = (data[:10], vardata[:10], data[10:],
                                    vardata[10:])

# rfx calibration
P = PT.permutation_test_twosample(data1, data2, XYZ)
c = [(P.random_Tvalues[P.ndraws * (0.95)], None)]
voxel_results, cluster_results, region_results = P.calibrate(nperms=100,
                                                             clusters=c)

# mfx calibration
P = PT.permutation_test_twosample(data1, data2, XYZ, vardata1=vardata1,
                                  vardata2=vardata2, stat_id="student_mfx")
voxel_results, cluster_results, region_results = P.calibrate(nperms=100,
                                                             clusters=c)

###############################################################################
# Print cluster statistics

level = 0.05

for results in cluster_results:
    nclust = results["labels"].max() + 1
    Tmax = np.zeros(nclust, float)
    Tmax_P = np.zeros(nclust, float)
    Diam = np.zeros(nclust, int)
    for j in range(nclust):
        I = np.where(results["labels"]==j)[0]
        Tmax[j] = P.Tvalues[I].max()
        Tmax_P[j] = voxel_results["Corr_p_values"][I].min()
        Diam[j]= PT.max_dist(XYZ, I, I)
    J = np.where(1 - (results["size_Corr_p_values"] > level) *
                     (results["Fisher_Corr_p_values"] > level) *
                     (Tmax_P > level))[0]
    print("\nDETECTED CLUSTERS STATISTICS:\n")
    print("Cluster detection threshold:", round(results["thresh"], 2))
    if results["diam"] != None:
        print("minimum cluster diameter", results["diam"])
    print("Cluster level FWER controled at", level)
    for j in J:
            X, Y, Z = results["peak_XYZ"][:, j]
            strXYZ = str(X).zfill(2) + " " + str(Y).zfill(2) + " " + \
                    str(Z).zfill(2)



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
"""
This script generates a noisy activation image and extracts the blob
from it.

This creates as output
- a label image representing the nested blobs,
- an image of the average signal per blob and
- an image with the terminal blob only

Author : Bertrand Thirion, 2009
"""
#autoindent

from os import mkdir, getcwd, path

from nibabel import load, save, Nifti1Image

import nipy.labs.spatial_models.hroi as hroi
from nipy.labs.spatial_models.discrete_domain import grid_domain_from_image

# Local import
from get_data_light import DATA_DIR, get_second_level_dataset


# data paths
input_image = path.join(DATA_DIR, 'spmT_0029.nii.gz')
if not path.exists(input_image):
    get_second_level_dataset()
write_dir = path.join(getcwd(), 'results')
if not path.exists(write_dir):
    mkdir(write_dir)

# parameters
threshold = 3.0 # blob-forming threshold
smin = 5 # size threshold on blobs

# prepare the data
nim = load(input_image)
mask_image = Nifti1Image((nim.get_data() ** 2 > 0).astype('u8'),
                         nim.get_affine())
domain = grid_domain_from_image(mask_image)
data = nim.get_data()
values = data[data != 0]

# compute the  nested roi object
nroi = hroi.HROI_as_discrete_domain_blobs(domain, values, threshold=threshold,
                                          smin=smin)

# compute region-level activation averages
activation = [values[nroi.select_id(id, roi=False)] for id in nroi.get_id()]
nroi.set_feature('activation', activation)
average_activation = nroi.representative_feature('activation')

# saving the blob image,i. e. a label image
descrip = "blob image extracted from %s" % input_image
wim = nroi.to_image('id', roi=True, descrip=descrip)
save(wim, path.join(write_dir, "blob.nii"))

# saving the image of the average-signal-per-blob
descrip = "blob average signal extracted from %s" % input_image
wim = nroi.to_image('activation', roi=True, descrip=descrip)
save(wim, path.join(write_dir, "bmap.nii"))

# saving the image of the end blobs or leaves
lroi = nroi.copy()
lroi.reduce_to_leaves()

descrip = "blob image extracted from %s" % input_image
wim = lroi.to_image('id', roi=True, descrip=descrip)
save(wim, path.join(write_dir, "leaves.nii"))

print("Wrote the blob image in %s" % path.join(write_dir, "blob.nii"))
print("Wrote the blob-average signal image in %s"
      % path.join(write_dir, "bmap.nii"))
print("Wrote the end-blob image in %s" % path.join(write_dir, "leaves.nii"))



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Intensity-based image registration
"""

import numpy as np

from ...core.image.image_spaces import (make_xyz_image,
                                        as_xyz_image,
                                        xyz_affine)

from .optimizer import configure_optimizer
from .affine import inverse_affine, subgrid_affine, affine_transforms
from .chain_transform import ChainTransform
from .similarity_measures import similarity_measures as _sms
from ._registration import _joint_histogram

MAX_INT = np.iinfo(np.intp).max

# Module globals
VERBOSE = True  # enables online print statements
OPTIMIZER = 'powell'
XTOL = 1e-2
FTOL = 1e-2
GTOL = 1e-3
MAXITER = 25
MAXFUN = None
CLAMP_DTYPE = 'short'  # do not edit
BINS = 256
SIMILARITY = 'crl1'
INTERP = 'pv'
NPOINTS = 64 ** 3

# Dictionary of interpolation methods (partial volume, trilinear,
# random)
interp_methods = {'pv': 0, 'tri': 1, 'rand': -1}


class HistogramRegistration(object):
    """
    A class to reprensent a generic intensity-based image registration
    algorithm.
    """
    def __init__(self, from_img, to_img,
                 from_bins=BINS, to_bins=None,
                 from_mask=None, to_mask=None,
                 similarity=SIMILARITY, interp=INTERP,
                 **kwargs):
        """
        Creates a new histogram registration object.

        Parameters
        ----------
        from_img : nipy-like image
          `From` image
        to_img : nipy-like image
          `To` image
        from_bins : integer
          Number of histogram bins to represent the `from` image
        to_bins : integer
          Number of histogram bins to represent the `to` image
        from_mask : array-like
          Mask to apply to the `from` image
        to_mask : array-like
          Mask to apply to the `to` image
        similarity : str or callable
          Cost-function for assessing image similarity. If a string,
          one of 'cc': correlation coefficient, 'cr': correlation
          ratio, 'crl1': L1-norm based correlation ratio, 'mi': mutual
          information, 'nmi': normalized mutual information, 'slr':
          supervised log-likelihood ratio. If a callable, it should
          take a two-dimensional array representing the image joint
          histogram as an input and return a float.
       interp : str
         Interpolation method.  One of 'pv': Partial volume, 'tri':
         Trilinear, 'rand': Random interpolation.  See ``joint_histogram.c``
        """
        # Function assumes xyx_affine for inputs
        from_img = as_xyz_image(from_img)
        to_img = as_xyz_image(to_img)

        # Binning sizes
        if to_bins is None:
            to_bins = from_bins

        # Clamping of the `from` image. The number of bins may be
        # overriden if unnecessarily large.
        data, from_bins = clamp(from_img.get_data(), bins=from_bins,
                                mask=from_mask)
        self._from_img = make_xyz_image(data, xyz_affine(from_img), 'scanner')
        # Set field of view in the `from` image with potential
        # subsampling for faster similarity evaluation. This also sets
        # the _from_data and _vox_coords attributes
        if from_mask is None:
            self.subsample(npoints=NPOINTS)
        else:
            corner, size = smallest_bounding_box(from_mask)
            self.set_fov(corner=corner, size=size, npoints=NPOINTS)

        # Clamping of the `to` image including padding with -1
        data, to_bins = clamp(to_img.get_data(), bins=to_bins,
                              mask=to_mask)
        self._to_data = -np.ones(np.array(to_img.shape) + 2, dtype=CLAMP_DTYPE)
        self._to_data[1:-1, 1:-1, 1:-1] = data
        self._to_inv_affine = inverse_affine(xyz_affine(to_img))

        # Joint histogram: must be double contiguous as it will be
        # passed to C routines which assume so
        self._joint_hist = np.zeros([from_bins, to_bins], dtype='double')

        # Set default registration parameters
        self._set_interp(interp)
        self._set_similarity(similarity, **kwargs)

    def _get_interp(self):
        return interp_methods.keys()\
            [interp_methods.values().index(self._interp)]

    def _set_interp(self, interp):
        self._interp = interp_methods[interp]

    interp = property(_get_interp, _set_interp)

    def set_fov(self, spacing=None, corner=(0, 0, 0), size=None,
                npoints=None):
        """
        Defines a subset of the `from` image to restrict joint
        histogram computation.

        Parameters
        ----------
        spacing : sequence (3,) of positive integers
          Subsampling of image in voxels, where None (default) results
          in the subsampling to be automatically adjusted to roughly
          match a cubic grid with `npoints` voxels
        corner : sequence (3,) of positive integers
          Bounding box origin in voxel coordinates
        size : sequence (3,) of positive integers
          Desired bounding box size
        npoints : positive integer
          Desired number of voxels in the bounding box. If a `spacing`
          argument is provided, then `npoints` is ignored.
        """
        if spacing is None and npoints is None:
            spacing = [1, 1, 1]
        if size is None:
            size = self._from_img.shape
        slicer = lambda c, s, sp:\
            tuple([slice(c[i], s[i] + c[i], sp[i]) for i in range(3)])
        # Adjust spacing to match desired field of view size
        if not spacing is None:
            fov_data = self._from_img.get_data()[slicer(corner, size, spacing)]
        else:
            fov_data = self._from_img.get_data()[slicer(corner, size, [1, 1, 1])]
            spacing = ideal_spacing(fov_data, npoints=npoints)
            fov_data = self._from_img.get_data()[slicer(corner, size, spacing)]
        self._from_data = fov_data
        self._from_npoints = (fov_data >= 0).sum()
        self._from_affine = subgrid_affine(xyz_affine(self._from_img),
                                           slicer(corner, size, spacing))
        # We cache the voxel coordinates of the clamped image
        self._vox_coords =\
            np.indices(self._from_data.shape).transpose((1, 2, 3, 0))

    def subsample(self, spacing=None, npoints=None):
        self.set_fov(spacing=spacing, npoints=npoints)

    def _set_similarity(self, similarity='cr', **kwargs):
        if similarity in _sms:
            self._similarity = similarity
            self._similarity_call =\
                _sms[similarity](self._joint_hist.shape, **kwargs)
        else:
            if not hasattr(similarity, '__call__'):
                raise ValueError('similarity should be callable')
            self._similarity = 'custom'
            self._similarity_call = similarity

    def _get_similarity(self):
        return self._similarity

    similarity = property(_get_similarity, _set_similarity)

    def eval(self, T):
        """
        Evaluate similarity function given a world-to-world transform.

        Parameters
        ----------
        T : Transform
            Transform object implementing ``apply`` method
        """
        Tv = ChainTransform(T, pre=self._from_affine, post=self._to_inv_affine)
        return self._eval(Tv)

    def _eval(self, Tv):
        """
        Evaluate similarity function given a voxel-to-voxel transform.

        Parameters
        ----------
        Tv : Transform
             Transform object implementing ``apply`` method
             Should map voxel space to voxel space
        """
        # trans_vox_coords needs be C-contiguous
        trans_vox_coords = Tv.apply(self._vox_coords)
        interp = self._interp
        if self._interp < 0:
            interp = - np.random.randint(MAX_INT)
        _joint_histogram(self._joint_hist,
                         self._from_data.flat,  # array iterator
                         self._to_data,
                         trans_vox_coords,
                         interp)
        return self._similarity_call(self._joint_hist)

    def optimize(self, T, optimizer=OPTIMIZER, **kwargs):
        """ Optimize transform `T` with respect to similarity measure.

        The input object `T` will change as a result of the optimization.

        Parameters
        ----------
        T : object or str
          An object representing a transformation that should
          implement ``apply`` method and ``param`` attribute or
          property. If a string, one of 'rigid', 'similarity', or
          'affine'. The corresponding transformation class is then
          initialized by default.
        optimizer : str
          Name of optimization function (one of 'powell', 'steepest',
          'cg', 'bfgs', 'simplex')
        **kwargs : dict
          keyword arguments to pass to optimizer
        """
        # Replace T if a string is passed
        if T in affine_transforms:
            T = affine_transforms[T]()

        # Pull callback out of keyword arguments, if present
        callback = kwargs.pop('callback', None)

        # Create transform chain object with T generating params
        Tv = ChainTransform(T, pre=self._from_affine, post=self._to_inv_affine)
        tc0 = Tv.param

        # Cost function to minimize
        def cost(tc):
            # This is where the similarity function is calculcated
            Tv.param = tc
            return -self._eval(Tv)

        # Callback during optimization
        if callback is None and VERBOSE:

            def callback(tc):
                Tv.param = tc
                print(Tv.optimizable)
                print(str(self.similarity) + ' = %s' % self._eval(Tv))
                print('')

        # Switching to the appropriate optimizer
        if VERBOSE:
            print('Initial guess...')
            print(Tv.optimizable)

        kwargs.setdefault('xtol', XTOL)
        kwargs.setdefault('ftol', FTOL)
        kwargs.setdefault('gtol', GTOL)
        kwargs.setdefault('maxiter', MAXITER)
        kwargs.setdefault('maxfun', MAXFUN)

        fmin, args, kwargs = configure_optimizer(optimizer,
                                                 fprime=None,
                                                 fhess=None,
                                                 **kwargs)

        # Output
        if VERBOSE:
            print ('Optimizing using %s' % fmin.__name__)
        kwargs['callback'] = callback
        Tv.param = fmin(cost, tc0, *args, **kwargs)
        return Tv.optimizable

    def explore(self, T0, *args):
        """
        Evaluate the similarity at the transformations specified by
        sequences of parameter values.

        For instance:

        explore(T0, (0, [-1,0,1]), (4, [-2.,2]))
        """
        nparams = T0.param.size
        deltas = [[0] for i in range(nparams)]
        for a in args:
            deltas[a[0]] = a[1]
        grids = np.mgrid[[slice(0, len(d)) for d in deltas]]
        ntrials = np.prod(grids.shape[1:])
        Deltas = [np.asarray(deltas[i])[grids[i, :]].ravel()\
                      for i in range(nparams)]
        simis = np.zeros(ntrials)
        params = np.zeros([nparams, ntrials])

        Tv = ChainTransform(T0, pre=self._from_affine,
                            post=self._to_inv_affine)
        param0 = Tv.param
        for i in range(ntrials):
            param = param0 + np.array([D[i] for D in Deltas])
            Tv.param = param
            simis[i] = self._eval(Tv)
            params[:, i] = param

        return simis, params


def _clamp(x, y, bins=BINS):

    # Threshold
    dmaxmax = 2 ** (8 * y.dtype.itemsize - 1) - 1
    dmax = bins - 1  # default output maximum value
    if dmax > dmaxmax:
        raise ValueError('Excess number of bins')
    xmin = float(x.min())
    xmax = float(x.max())
    d = xmax - xmin

    """
    If the image dynamic is small, no need for compression: just
    downshift image values and re-estimate the dynamic range (hence
    xmax is translated to xmax-tth casted to the appropriate
    dtype. Otherwise, compress after downshifting image values (values
    equal to the threshold are reset to zero).
    """
    if issubclass(x.dtype.type, np.integer) and d <= dmax:
        y[:] = x - xmin
        bins = int(d) + 1
    else:
        a = dmax / d
        y[:] = np.round(a * (x - xmin))

    return y, bins


def clamp(x, bins=BINS, mask=None):
    """
    Clamp array values that fall within a given mask in the range
    [0..bins-1] and reset masked values to -1.

    Parameters
    ----------
    x : ndarray
      The input array

    bins : number
      Desired number of bins

    mask : ndarray, tuple or slice
      Anything such that x[mask] is an array.

    Returns
    -------
    y : ndarray
      Clamped array, masked items are assigned -1

    bins : number
      Adjusted number of bins

    """
    if bins > np.iinfo(np.short).max:
        raise ValueError('Too large a bin size')
    y = -np.ones(x.shape, dtype=CLAMP_DTYPE)
    if mask is None:
        y, bins = _clamp(x, y, bins)
    else:
        ym = y[mask]
        xm = x[mask]
        ym, bins = _clamp(xm, ym, bins)
        y[mask] = ym
    return y, bins


def ideal_spacing(data, npoints):
    """
    Tune spacing factors so that the number of voxels in the
    output block matches a given number.

    Parameters
    ----------
    data : ndarray or sequence
      Data image to subsample

    npoints : number
      Target number of voxels (negative values will be ignored)

    Returns
    -------
    spacing: ndarray
      Spacing factors

    """
    dims = data.shape
    actual_npoints = (data >= 0).sum()
    spacing = np.ones(3, dtype='uint')

    while actual_npoints > npoints:

        # Subsample the direction with the highest number of samples
        ddims = dims / spacing
        if ddims[0] >= ddims[1] and ddims[0] >= ddims[2]:
            dir = 0
        elif ddims[1] > ddims[0] and ddims[1] >= ddims[2]:
            dir = 1
        else:
            dir = 2
        spacing[dir] += 1
        subdata = data[::spacing[0], ::spacing[1], ::spacing[2]]
        actual_npoints = (subdata >= 0).sum()

    return spacing


def smallest_bounding_box(msk):
    """
    Extract the smallest bounding box from a mask
    """
    x, y, z = np.where(msk > 0)
    corner = [x.min(), y.min(), z.min()]
    size = [x.max() + 1, y.max() + 1, z.max() + 1]
    return corner, size



"""
Nose test running.

This module implements ``test()`` and ``bench()`` functions for NumPy modules.

"""
import os
import sys

def get_package_name(filepath):
    """
    Given a path where a package is installed, determine its name.

    Parameters
    ----------
    filepath : str
        Path to a file. If the determination fails, "numpy" is returned.

    Examples
    --------
    >>> np.testing.nosetester.get_package_name('nonsense')
    'numpy'

    """

    fullpath = filepath[:]
    pkg_name = []
    while 'site-packages' in filepath or 'dist-packages' in filepath:
        filepath, p2 = os.path.split(filepath)
        if p2 in ('site-packages', 'dist-packages'):
            break
        pkg_name.append(p2)

    # if package name determination failed, just default to numpy/scipy
    if not pkg_name:
        if 'scipy' in fullpath:
            return 'scipy'
        else:
            return 'numpy'

    # otherwise, reverse to get correct order and return
    pkg_name.reverse()

    # don't include the outer egg directory
    if pkg_name[0].endswith('.egg'):
        pkg_name.pop(0)

    return '.'.join(pkg_name)

def import_nose():
    """ Import nose only when needed.
    """
    fine_nose = True
    minimum_nose_version = (0,10,0)
    try:
        import nose
        from nose.tools import raises
    except ImportError:
        fine_nose = False
    else:
        if nose.__versioninfo__ < minimum_nose_version:
            fine_nose = False

    if not fine_nose:
        msg = 'Need nose >= %d.%d.%d for tests - see ' \
              'http://somethingaboutorange.com/mrl/projects/nose' % \
              minimum_nose_version

        raise ImportError(msg)

    return nose

def run_module_suite(file_to_run = None):
    if file_to_run is None:
        f = sys._getframe(1)
        file_to_run = f.f_locals.get('__file__', None)
        if file_to_run is None:
            raise AssertionError

    import_nose().run(argv=['',file_to_run])


class NoseTester(object):
    """
    Nose test runner.

    This class is made available as numpy.testing.Tester, and a test function
    is typically added to a package's __init__.py like so::

      from numpy.testing import Tester
      test = Tester().test

    Calling this test function finds and runs all tests associated with the
    package and all its sub-packages.

    Attributes
    ----------
    package_path : str
        Full path to the package to test.
    package_name : str
        Name of the package to test.

    Parameters
    ----------
    package : module, str or None
        The package to test. If a string, this should be the full path to
        the package. If None (default), `package` is set to the module from
        which `NoseTester` is initialized.

    """
    # Stuff to exclude from tests. These are from numpy.distutils
    excludes = ['f2py_ext',
                'f2py_f90_ext',
                'gen_ext',
                'pyrex_ext',
                'swig_ext']

    def __init__(self, package=None):
        ''' Test class init

        Parameters
        ----------
        package : string or module
            If string, gives full path to package
            If None, extract calling module path
            Default is None
        '''
        package_name = None
        if package is None:
            f = sys._getframe(1)
            package_path = f.f_locals.get('__file__', None)
            if package_path is None:
                raise AssertionError
            package_path = os.path.dirname(package_path)
            package_name = f.f_locals.get('__name__', None)
        elif isinstance(package, type(os)):
            package_path = os.path.dirname(package.__file__)
            package_name = getattr(package, '__name__', None)
        else:
            package_path = str(package)

        self.package_path = package_path

        # find the package name under test; this name is used to limit coverage
        # reporting (if enabled)
        if package_name is None:
            package_name = get_package_name(package_path)
        self.package_name = package_name

    def _test_argv(self, label, verbose, extra_argv):
        ''' Generate argv for nosetest command

        Parameters
        ----------
        label : {'fast', 'full', '', attribute identifier}, optional
            see ``test`` docstring
        verbose : int, optional
            Verbosity value for test outputs, in the range 1-10. Default is 1.
        extra_argv : list, optional
            List with any extra arguments to pass to nosetests.

        Returns
        -------
        argv : list
            command line arguments that will be passed to nose
        '''
        argv = [__file__, self.package_path, '-s']
        if label and label != 'full':
            if not isinstance(label, basestring):
                raise TypeError('Selection label should be a string')
            if label == 'fast':
                label = 'not slow'
            argv += ['-A', label]
        argv += ['--verbosity', str(verbose)]
        if extra_argv:
            argv += extra_argv
        return argv

    def _show_system_info(self):
        nose = import_nose()

        import numpy
        print "NumPy version %s" % numpy.__version__
        npdir = os.path.dirname(numpy.__file__)
        print "NumPy is installed in %s" % npdir

        if 'scipy' in self.package_name:
            import scipy
            print "SciPy version %s" % scipy.__version__
            spdir = os.path.dirname(scipy.__file__)
            print "SciPy is installed in %s" % spdir

        pyversion = sys.version.replace('\n','')
        print "Python version %s" % pyversion
        print "nose version %d.%d.%d" % nose.__versioninfo__

    def _get_custom_doctester(self):
        """ Return instantiated plugin for doctests

        Allows subclassing of this class to override doctester

        A return value of None means use the nose builtin doctest plugin
        """
        from noseclasses import NumpyDoctest
        return NumpyDoctest()

    def prepare_test_args(self, label='fast', verbose=1, extra_argv=None,
                          doctests=False, coverage=False):
        """
        Run tests for module using nose.

        This method does the heavy lifting for the `test` method. It takes all
        the same arguments, for details see `test`.

        See Also
        --------
        test

        """
        # fail with nice error message if nose is not present
        import_nose()
        # compile argv
        argv = self._test_argv(label, verbose, extra_argv)
        # bypass tests noted for exclude
        for ename in self.excludes:
            argv += ['--exclude', ename]
        # our way of doing coverage
        if coverage:
            argv+=['--cover-package=%s' % self.package_name, '--with-coverage',
                   '--cover-tests', '--cover-inclusive', '--cover-erase']
        # construct list of plugins
        import nose.plugins.builtin
        from noseclasses import KnownFailure, Unplugger
        plugins = [KnownFailure()]
        plugins += [p() for p in nose.plugins.builtin.plugins]
        # add doctesting if required
        doctest_argv = '--with-doctest' in argv
        if doctests == False and doctest_argv:
            doctests = True
        plug = self._get_custom_doctester()
        if plug is None:
            # use standard doctesting
            if doctests and not doctest_argv:
                argv += ['--with-doctest']
        else: # custom doctesting
            if doctest_argv: # in fact the unplugger would take care of this
                argv.remove('--with-doctest')
            plugins += [Unplugger('doctest'), plug]
            if doctests:
                argv += ['--with-' + plug.name]
        return argv, plugins

    def test(self, label='fast', verbose=1, extra_argv=None, doctests=False,
             coverage=False):
        """
        Run tests for module using nose.

        Parameters
        ----------
        label : {'fast', 'full', '', attribute identifier}, optional
            Identifies the tests to run. This can be a string to pass to
            the nosetests executable with the '-A' option, or one of several
            special values.  Special values are:
            * 'fast' - the default - which corresponds to the ``nosetests -A``
              option of 'not slow'.
            * 'full' - fast (as above) and slow tests as in the
              'no -A' option to nosetests - this is the same as ''.
            * None or '' - run all tests.
            attribute_identifier - string passed directly to nosetests as '-A'.
        verbose : int, optional
            Verbosity value for test outputs, in the range 1-10. Default is 1.
        extra_argv : list, optional
            List with any extra arguments to pass to nosetests.
        doctests : bool, optional
            If True, run doctests in module. Default is False.
        coverage : bool, optional
            If True, report coverage of NumPy code. Default is False.
            (This requires the `coverage module:
             <http://nedbatchelder.com/code/modules/coverage.html>`_).

        Returns
        -------
        result : object
            Returns the result of running the tests as a
            ``nose.result.TextTestResult`` object.

        Notes
        -----
        Each NumPy module exposes `test` in its namespace to run all tests for it.
        For example, to run all tests for numpy.lib:

        >>> np.lib.test() #doctest: +SKIP

        Examples
        --------
        >>> result = np.lib.test() #doctest: +SKIP
        Running unit tests for numpy.lib
        ...
        Ran 976 tests in 3.933s

        OK

        >>> result.errors #doctest: +SKIP
        []
        >>> result.knownfail #doctest: +SKIP
        []
        """

        # cap verbosity at 3 because nose becomes *very* verbose beyond that
        verbose = min(verbose, 3)

        import utils
        utils.verbose = verbose

        if doctests:
            print "Running unit tests and doctests for %s" % self.package_name
        else:
            print "Running unit tests for %s" % self.package_name

        self._show_system_info()

        # reset doctest state on every run
        import doctest
        doctest.master = None

        argv, plugins = self.prepare_test_args(label, verbose, extra_argv,
                                               doctests, coverage)
        from noseclasses import NumpyTestProgram
        t = NumpyTestProgram(argv=argv, exit=False, plugins=plugins)
        return t.result

    def bench(self, label='fast', verbose=1, extra_argv=None):
        """
        Run benchmarks for module using nose.

        Parameters
        ----------
        label : {'fast', 'full', '', attribute identifier}, optional
            Identifies the benchmarks to run. This can be a string to pass to
            the nosetests executable with the '-A' option, or one of several
            special values.  Special values are:
            * 'fast' - the default - which corresponds to the ``nosetests -A``
              option of 'not slow'.
            * 'full' - fast (as above) and slow benchmarks as in the
              'no -A' option to nosetests - this is the same as ''.
            * None or '' - run all tests.
            attribute_identifier - string passed directly to nosetests as '-A'.
        verbose : int, optional
            Verbosity value for benchmark outputs, in the range 1-10. Default is 1.
        extra_argv : list, optional
            List with any extra arguments to pass to nosetests.

        Returns
        -------
        success : bool
            Returns True if running the benchmarks works, False if an error
            occurred.

        Notes
        -----
        Benchmarks are like tests, but have names starting with "bench" instead
        of "test", and can be found under the "benchmarks" sub-directory of the
        module.

        Each NumPy module exposes `bench` in its namespace to run all benchmarks
        for it.

        Examples
        --------
        >>> success = np.lib.bench() #doctest: +SKIP
        Running benchmarks for numpy.lib
        ...
        using 562341 items:
        unique:
        0.11
        unique1d:
        0.11
        ratio: 1.0
        nUnique: 56230 == 56230
        ...
        OK

        >>> success #doctest: +SKIP
        True

        """

        print "Running benchmarks for %s" % self.package_name
        self._show_system_info()

        argv = self._test_argv(label, verbose, extra_argv)
        argv += ['--match', r'(?:^|[\\b_\\.%s-])[Bb]ench' % os.sep]

        # import nose or make informative error
        nose = import_nose()

        # get plugin to disable doctests
        from noseclasses import Unplugger
        add_plugins = [Unplugger('doctest')]

        return nose.run(argv=argv, addplugins=add_plugins)




#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" Example of FIR model using formula framework

Shows how to use B splines as basis functions for the FIR instead of simple
boxcars.

Requires matplotlib
"""

import numpy as np

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

from nipy.fixes.sympy.utilities.lambdify import implemented_function
from nipy.algorithms.statistics.api import Formula
from nipy.modalities.fmri import utils

def linBspline(knots):
    """ Create linear B spline that is zero outside [knots[0], knots[-1]]

    (knots is assumed to be sorted).
    """
    fns = []
    knots = np.array(knots)
    for i in range(knots.shape[0]-2):
        name = 'bs_%s' % i
        k1, k2, k3 = knots[i:i+3]
        d1 = k2-k1
        def anon(x,k1=k1,k2=k2,k3=k3):
            return ((x-k1) / d1 * np.greater(x, k1) * np.less_equal(x, k2) +
                    (k3-x) / d1 * np.greater(x, k2) * np.less(x, k3))
        fns.append(implemented_function(name, anon))
    return fns


# The splines are functions of t (time)
bsp_fns = linBspline(np.arange(0,10,2))

# We're going to evaluate at these specific values of time
tt = np.linspace(0,50,101)
tvals= tt.view(np.dtype([('t', np.float)]))

# Some inter-stimulus intervals
isis = np.random.uniform(low=0, high=3, size=(4,)) + 10.

# Made into event onset times
e = np.cumsum(isis)

# Make event onsets into functions of time convolved with the spline functions.
event_funcs = [utils.events(e, f=fn) for fn in bsp_fns]

# Put into a formula.
f = Formula(event_funcs)

# The design matrix
X = f.design(tvals, return_float=True)

# Show the design matrix as line plots
plt.plot(X[:,0])
plt.plot(X[:,1])
plt.plot(X[:,2])
plt.xlabel('time (s)')
plt.title('B spline used as bases for an FIR response model')
plt.show()



import numpy as np

from .segmentation import (Segmentation,
                           moment_matching,
                           map_from_ppm)

T1_ref_params = {}
T1_ref_params['glob_mu'] = 1643.2
T1_ref_params['glob_sigma'] = 252772.3
T1_ref_params['3k'] = {
    'mu': np.array([813.9, 1628.3, 2155.8]),
    'sigma': np.array([46499.0, 30233.4, 17130.0])}
T1_ref_params['4k'] = {
    'mu': np.array([816.1, 1613.7, 1912.3, 2169.3]),
    'sigma': np.array([47117.6, 27053.8, 8302.2, 14970.8])}
T1_ref_params['5k'] = {
    'mu': np.array([724.2, 1169.3, 1631.5, 1917.0, 2169.2]),
    'sigma': np.array([22554.8, 21368.9, 20560.1, 7302.6, 14962.1])}


class BrainT1Segmentation(object):

    def __init__(self, data, mask=None, model='3k',
                 niters=25, ngb_size=6, beta=0.5,
                 ref_params=None, init_params=None,
                 convert=True):

        self.labels = ('CSF', 'GM', 'WM')
        self.data = data
        self.mask = mask

        mixmat = np.asarray(model)
        if mixmat.ndim == 2:
            nclasses = mixmat.shape[0]
            if nclasses < 3:
                raise ValueError('at least 3 classes required')
            if not mixmat.shape[1] == 3:
                raise ValueError('mixing matrix should have 3 rows')
            self.mixmat = mixmat
        elif model == '3k':
            self.mixmat = np.eye(3)
        elif model == '4k':
            self.mixmat = np.array([[1., 0., 0.],
                                    [0., 1., 0.],
                                    [0., 1., 0.],
                                    [0., 0., 1.]])
        elif model == '5k':
            self.mixmat = np.array([[1., 0., 0.],
                                    [1., 0., 0.],
                                    [0., 1., 0.],
                                    [0., 1., 0.],
                                    [0., 0., 1.]])
        else:
            raise ValueError('unknown brain segmentation model')

        self.niters = int(niters)
        self.beta = float(beta)
        self.ngb_size = int(ngb_size)

        # Class parameter initialization
        if init_params == None:
            if ref_params == None:
                ref_params = T1_ref_params
            self.init_mu, self.init_sigma = self._init_parameters(ref_params)
        else:
            self.init_mu = np.array(init_params[0], dtype='double')
            self.init_sigma = np.array(init_params[1], dtype='double')
            if not len(self.init_mu) == self.mixmat.shape[0]\
                    or not len(self.init_sigma) == self.mixmat.shape[0]:
                raise ValueError('Inconsistent initial parameter estimates')

        self._run()
        if convert:
            self.convert()
        else:
            self.label = map_from_ppm(self.ppm, self.mask)

    def _init_parameters(self, ref_params):

        if not self.mask == None:
            data = self.data[self.mask]
        else:
            data = self.data

        nclasses = self.mixmat.shape[0]
        if nclasses <= 5:
            key = str(self.mixmat.shape[0]) + 'k'
            ref_mu = ref_params[key]['mu']
            ref_sigma = ref_params[key]['sigma']
        else:
            ref_mu = np.linspace(ref_params['3k']['mu'][0],
                                 ref_params['3k']['mu'][-1],
                                 num=nclasses)
            ref_sigma = np.linspace(ref_params['3k']['sigma'][0],
                                    ref_params['3k']['sigma'][-1],
                                    num=nclasses)

        return moment_matching(data, ref_mu, ref_sigma,
                               ref_params['glob_mu'],
                               ref_params['glob_sigma'])

    def _run(self):
        S = Segmentation(self.data, mask=self.mask,
                         mu=self.init_mu, sigma=self.init_sigma,
                         ngb_size=self.ngb_size, beta=self.beta)
        S.run(niters=self.niters)
        self.mu = S.mu
        self.sigma = S.sigma
        self.ppm = S.ppm

    def convert(self):
        if self.ppm.shape[-1] == self.mixmat.shape[0]:
            self.ppm = np.dot(self.ppm, self.mixmat)
            self.label = map_from_ppm(self.ppm, self.mask)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from numpy.testing import assert_equal, assert_almost_equal
import numpy as np

from .. import routines, onesample, twosample

def slow_add_lines(A, B, I):
    for i in xrange(len(I)):
        B[I[i]] += A[i]


def test_add_lines():
    nlines = 100000
    ncols = 1
    A = np.random.rand(nlines, ncols)
    B = np.random.rand(nlines, ncols)
    I = np.asarray(np.random.randint(nlines, size=nlines)
            ).astype(np.int)
    C1 = B.copy()
    C2 = B.copy()
    routines.add_lines(A, C1, I)
    slow_add_lines(A, C2, I)
    assert_almost_equal(C1, C2)         

def test_add_lines2():
    nlines = 1e5
    ncols = 1
    A = np.random.rand(nlines, ncols)
    B = np.random.rand(nlines, ncols)
    I = np.asarray(np.random.randint(nlines, size=nlines)
            ).astype(np.int)
    C1 = B.copy()
    C2 = B.copy()
    routines.add_lines(A, C1, I)
    slow_add_lines(A, C2, I)
    assert_almost_equal(C1, C2)


def test_onesample_stat():
    dx, dy, dz = 3, 4, 2
    nvox = dx*dy*dz
    nsub = 12
    # Make surrogate data 
    aux = np.arange(nvox)
    x = np.reshape(aux.repeat(nsub), [dx, dy, dz, nsub])
    # Gold standard 
    y_target = np.inf * np.ones(nvox)
    y_target[0] = 0.0
    # Test: input C-contiguous, data owner, axis=3
    y = onesample.stat(x, axis=3).reshape(nvox)
    assert_equal(y, y_target)
    # Test: input F-contiguous, not owner, axis=0 
    y = onesample.stat(x.T, axis=0).reshape(nvox)
    assert_equal(y, y_target)
    # Test: input C-contiguous, data owner, axis=0
    xT = x.T.copy()
    y = onesample.stat(xT, axis=0).reshape(nvox)
    assert_equal(y, y_target)
    

    
if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
''' General matrix and other utilities for statistics '''

import numpy as np
import scipy.interpolate

__docformat__ = 'restructuredtext'


def mad(a, c=0.6745, axis=0):
    """
    Median Absolute Deviation:

    median(abs(a - median(a))) / c
    """
    _shape = a.shape
    a.shape = np.product(a.shape, axis=0)
    m = np.median(np.fabs(a - np.median(a))) / c
    a.shape = _shape
    return m


class StepFunction(object):
    """
    A basic step function: values at the ends are handled in the simplest
    way possible: everything to the left of x[0] is set to ival; everything
    to the right of x[-1] is set to y[-1].

    Examples
    --------
    >>> x = np.arange(20)
    >>> y = np.arange(20)
    >>> f = StepFunction(x, y)
    >>>
    >>> print f(3.2)
    3.0
    >>> print f([[3.2,4.5],[24,-3.1]])
    [[  3.   4.]
     [ 19.   0.]]
    """

    def __init__(self, x, y, ival=0., sorted=False):

        _x = np.asarray(x)
        _y = np.asarray(y)

        if _x.shape != _y.shape:
            raise ValueError(
                'in StepFunction: x and y do not have the same shape')
        if len(_x.shape) != 1:
            raise ValueError('in StepFunction: x and y must be 1-dimensional')

        self.x = np.hstack([[- np.inf], _x])
        self.y = np.hstack([[ival], _y])

        if not sorted:
            asort = np.argsort(self.x)
            self.x = np.take(self.x, asort, 0)
            self.y = np.take(self.y, asort, 0)
        self.n = self.x.shape[0]

    def __call__(self, time):
        tind = np.searchsorted(self.x, time) - 1
        return self.y[tind]


def ECDF(values):
    """
    Return the ECDF of an array as a step function.
    """
    x = np.array(values, copy=True)
    x.sort()
    x.shape = np.product(x.shape, axis=0)
    n = x.shape[0]
    y = (np.arange(n) + 1.) / n
    return StepFunction(x, y)


def monotone_fn_inverter(fn, x, vectorized=True, **keywords):
    """
    Given a monotone function x (no checking is done to verify monotonicity)
    and a set of x values, return an linearly interpolated approximation
    to its inverse from its values on x.
    """

    if vectorized:
        y = fn(x, **keywords)
    else:
        y = []
        for _x in x:
            y.append(fn(_x, **keywords))
        y = np.array(y)

    a = np.argsort(y)

    return scipy.interpolate.interp1d(y[a], x[a])



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
This module defines the two default GLM passes of fmristat

The results of both passes of the GLM get pushed around by generators, which
know how to get out the (probably 3D) data for each slice, or parcel (for the
AR) case, estimate in 2D, then store the data back again in its original shape.

The containers here, in the execute methods, know how to reshape the data on the
way into the estimation (to 2D), then back again, to 3D, or 4D.

It's relatively easy to do this when just iterating over simple slices, but it
gets a bit more complicated when taking arbitrary shaped samples from the image,
as we do for estimating the AR coefficients, where we take all the voxels with
similar AR coefficients at once.
"""

import copy

import os.path as path

import numpy as np
import scipy.linalg as spl

from nipy.algorithms.statistics.models.regression import (
    OLSModel, ARModel, ar_bias_corrector, ar_bias_correct)
from nipy.algorithms.statistics.formula import make_recarray

# nipy core imports
from nipy.core.api import Image, parcels, matrix_generator, AffineTransform

# nipy IO imports
from nipy.io.api import save_image

# fmri imports
from ..api import FmriImageList, axis0_generator

from . import outputters


class ModelOutputImage(object):
    """
    These images have their values filled in as the model is fit, and
    are saved to disk after being completely filled in.

    They are saved to disk by calling the 'save' method.

    The __getitem__ and __setitem__ calls are delegated to a private
    Image.  An exception is raised if trying to get/set data after the
    data has been saved to disk.
    """

    def __init__(self, filename, coordmap, shape, clobber=False):
        self.filename = filename
        self._im_data = np.zeros(shape)
        self._im = Image(self._im_data, coordmap)
        # Using a dangerous undocumented API here
        self.clobber = clobber
        self._flushed = False

    def save(self):
        """
        Save current Image data to disk
        """
        if not self.clobber and path.exists(self.filename):
            raise ValueError('trying to clobber existing file')
        save_image(self._im, self.filename)
        self._flushed = True
        del(self._im)

    def __getitem__(self, item):
        if self._flushed:
            raise ValueError('trying to read value from a '
                             'saved ModelOutputImage')
        return self._im_data[item]

    def __setitem__(self, item, value):
        if self._flushed:
            raise ValueError('trying to set value on saved'
                             'ModelOutputImage')
        self._im_data[item] = value


def model_generator(formula, data, volume_start_times, iterable=None,
                    slicetimes=None, model_type=OLSModel,
                    model_params = lambda x: ()):
    """
    Generator for the models for a pass of fmristat analysis.
    """
    volume_start_times = make_recarray(volume_start_times.astype(float), 't')
    # Generator for slices of the data with time as first axis
    axis0_gen = axis0_generator(data, slicers=iterable)
    # Iterate over 2D slices of the data
    for indexer, indexed_data in matrix_generator(axis0_gen):
        model_args = model_params(indexer) # model may depend on i
        # Get the design for these volume start times
        design = formula.design(volume_start_times, return_float=True)
        # Make the model from the design
        rmodel = model_type(design, *model_args)
        yield indexer, indexed_data, rmodel


def results_generator(model_iterable):
    """
    Generator for results from an iterator that returns
    (index, data, model) tuples.

    See model_generator.
    """
    for i, d, m in model_iterable:
        yield i, m.fit(d)


class OLS(object):
    """
    First pass through fmri_image.

    Parameters
    ----------
    fmri_image : `FmriImageList` or 4D image
       object returning 4D data from np.asarray, with first
       (``object[0]``) axis being the independent variable of the model;
       object[0] returns an object with attribute ``shape``.
    formula :  :class:`nipy.algorithms.statistics.formula.Formula`
    outputs :
    volume_start_times :
    """

    def __init__(self, fmri_image, formula, outputs=[],
                 volume_start_times=None):
        self.fmri_image = fmri_image
        try:
            self.data = fmri_image.get_data()
        except AttributeError:
            self.data = fmri_image.get_list_data(axis=0)
        self.formula = formula
        self.outputs = outputs
        if volume_start_times is None:
            self.volume_start_times = self.fmri_image.volume_start_times
        else:
            self.volume_start_times = volume_start_times

    def execute(self):
        m = model_generator(self.formula, self.data,
                            self.volume_start_times,
                            model_type=OLSModel)
        r = results_generator(m)

        def reshape(i, x):
            if len(x.shape) == 2:
                if type(i) is type(1):
                    x.shape = (x.shape[0],) + self.fmri_image[0].shape[1:]
                if type(i) not in [type([]), type(())]:
                    i = (i,)
                else:
                    i = tuple(i)
                i = (slice(None,None,None),) + tuple(i)
            else:
                if type(i) is type(1):
                    x.shape = self.fmri_image[0].shape[1:]
            return i, x

        o = generate_output(self.outputs, r, reshape=reshape)


def estimateAR(resid, design, order=1):
    """
    Estimate AR parameters using bias correction from fMRIstat.

    Parameters
    ----------
    resid:  array-like
        residuals from model
    model:  an OLS model used to estimate residuals

    Returns
    -------
    output : array
        shape (order, resid
    """
    invM = ar_bias_corrector(design, spl.pinv(design), order)
    return ar_bias_correct(resid, order, invM)


class AR1(object):
    """
    Second pass through fmri_image.

    Parameters
    ----------
    fmri_image : `FmriImageList`
       object returning 4D array from ``np.asarray``, having attribute
       ``volume_start_times`` (if `volume_start_times` is None), and
       such that ``object[0]`` returns something with attributes ``shape``
    formula :  :class:`nipy.algorithms.statistics.formula.Formula`
    rho : ``Image``
       image of AR(1) coefficients.  Returning data from
       ``rho.get_data()``, and having attribute ``coordmap``
    outputs :
    volume_start_times : 
    """

    def __init__(self, fmri_image, formula, rho, outputs=[],
                 volume_start_times=None):
        self.fmri_image = fmri_image
        try:
            self.data = fmri_image.get_data()
        except AttributeError:
            self.data = fmri_image.get_list_data(axis=0)
        self.formula = formula
        self.outputs = outputs
        # Cleanup rho values, truncate them to a scale of 0.01
        g = copy.copy(rho.coordmap)
        rho = rho.get_data()
        m = np.isnan(rho)
        r = (np.clip(rho,-1,1) * 100).astype(np.int) / 100.
        r[m] = np.inf
        self.rho = Image(r, g)
        if volume_start_times is None:
            self.volume_start_times = self.fmri_image.volume_start_times
        else:
            self.volume_start_times = volume_start_times

    def execute(self):

        iterable = parcels(self.rho, exclude=[np.inf])
        def model_params(i):
            return (self.rho.get_data()[i].mean(),)
        # Generates indexer, data, model
        m = model_generator(self.formula, self.data,
                            self.volume_start_times,
                            iterable=iterable,
                            model_type=ARModel,
                            model_params=model_params)
        # Generates indexer, data, 2D results
        r = results_generator(m)

        def reshape(i, x):
            """
            To write output, arrays have to be reshaped --
            this function does the appropriate reshaping for the two
            passes of fMRIstat.

            These passes are:
              i) 'slices through the z-axis'
              ii) 'parcels of approximately constant AR1 coefficient'
            """
            if len(x.shape) == 2: # 2D imput matrix
                if type(i) is type(1): # integer indexing
                    # reshape to ND (where N is probably 4)
                    x.shape = (x.shape[0],) + self.fmri_image[0].shape[1:]
                # Convert lists to tuples, put anything else into a tuple
                if type(i) not in [type([]), type(())]:
                    i = (i,)
                else:
                    i = tuple(i)
                # Add : to indexing
                i = (slice(None,None,None),) + tuple(i)
            else: # not 2D
                if type(i) is type(1): # integer indexing
                    x.shape = self.fmri_image[0].shape[1:]
            return i, x

        # Put results pulled from results generator r, into outputs
        o = generate_output(self.outputs, r, reshape=reshape)


def output_T(outbase, contrast, fmri_image, effect=True, sd=True, t=True,
             clobber=False):
    """ Return t contrast regression outputs list for `contrast`

    Parameters
    ----------
    outbase : string
        Base filename that will be used to construct a set of files
        for the TContrast.  For example, outbase='output.nii' will
        result in the following files (assuming defaults for all other
        params): output_effect.nii, output_sd.nii, output_t.nii
    contrast : array
        F contrast matrix
    fmri_image : ``FmriImageList`` or ``Image``
        object such that ``object[0]`` has attributes ``shape`` and
        ``coordmap``
    effect : {True, False}, optional
        whether to write an effect image
    sd : {True, False}, optional
        whether to write a standard deviation image
    t : {True, False}, optional
        whether to write a t image
    clobber : {False, True}, optional
        whether to overwrite images that exist.

    Returns
    -------
    reglist : ``RegressionOutputList`` instance
        Regression output list with selected outputs, where selection is by
        inputs `effect`, `sd` and `t`

    Notes
    -----
    Note that this routine uses the corresponding ``output_T`` routine in
    :mod:`outputters`, but indirectly via the TOutput object.
    """
    def build_filename(label):
        index = outbase.find('.')
        return ''.join([outbase[:index], '_', label, outbase[index:]])
    if effect:
        effectim = ModelOutputImage(build_filename('effect'),
                                    fmri_image[0].coordmap,
                                    fmri_image[0].shape, clobber=clobber)
    else:
        effectim = None
    if sd:
        sdim = ModelOutputImage(build_filename('sd'),
                                fmri_image[0].coordmap, fmri_image[0].shape,
                                clobber=clobber)
    else:
        sdim = None
    if t:
        tim = ModelOutputImage(build_filename('t'),
                               fmri_image[0].coordmap,fmri_image[0].shape,
                               clobber=clobber)
    else:
        tim = None
    return outputters.TOutput(contrast, effect=effectim, sd=sdim, t=tim)


def output_F(outfile, contrast, fmri_image, clobber=False):
    ''' output F statistic images

    Parameters
    ----------
    outfile : str
        filename for F contrast image
    contrast : array
        F contrast matrix
    fmri_image : ``FmriImageList`` or ``Image``
        object such that ``object[0]`` has attributes ``shape`` and
        ``coordmap``
    clobber : bool
        if True, overwrites previous output; if False, raises error

    Returns
    -------
    f_reg_out : ``RegressionOutput`` instance
        Object that can a) be called with a results instance as argument,
        returning an array, and b) accept the output array for storing, via
        ``obj[slice_spec] = arr`` type slicing.
    '''
    f = ModelOutputImage(outfile, fmri_image[0].coordmap, fmri_image[0].shape,
                         clobber=clobber)
    return outputters.RegressionOutput(f, lambda x:
                                       outputters.output_F(x, contrast))


def output_AR1(outfile, fmri_image, clobber=False):
    """
    Create an output file of the AR1 parameter from the OLS pass of
    fmristat.

    Parameters
    ----------
    outfile :
    fmri_image : ``FmriImageList`` or 4D image
       object such that ``object[0]`` has attributes ``coordmap`` and ``shape``
    clobber : bool
       if True, overwrite previous output

    Returns
    -------
    regression_output : ``RegressionOutput`` instance
    """
    outim = ModelOutputImage(outfile, fmri_image[0].coordmap,
                             fmri_image[0].shape, clobber=clobber)
    return outputters.RegressionOutput(outim, outputters.output_AR1)


def output_resid(outfile, fmri_image, clobber=False):
    """
    Create an output file of the residuals parameter from the OLS pass of
    fmristat.

    Uses affine part of the first image to output resids unless
    fmri_image is an Image.

    Parameters
    ----------
    outfile :
    fmri_image : ``FmriImageList`` or 4D image
       If ``FmriImageList``, needs attributes ``volume_start_times``,
       supports len(), and object[0] has attributes ``affine``,
       ``coordmap`` and ``shape``, from which we create a new 4D
       coordmap and shape
       If 4D image, use the images coordmap and shape
    clobber : bool
       if True, overwrite previous output

    Returns
    -------
    regression_output :
    """
    if isinstance(fmri_image, FmriImageList):
        n = len(fmri_image.list)
        T = np.zeros((5,5))
        g = fmri_image[0].coordmap
        T[1:,1:] = fmri_image[0].affine
        T[0,0] = (fmri_image.volume_start_times[1:] -
                  fmri_image.volume_start_times[:-1]).mean()
        # FIXME: NIFTI specific naming here
        innames = ["t"] + list(g.function_domain.coord_names)
        outnames = ["t"] + list(g.function_range.coord_names)
        cmap = AffineTransform.from_params(innames, outnames, T)
        shape = (n,) + fmri_image[0].shape
    elif isinstance(fmri_image, Image):
        cmap = fmri_image.coordmap
        shape = fmri_image.shape
    else:
        raise ValueError, "expecting FmriImageList or 4d Image"

    outim = ModelOutputImage(outfile, cmap, shape, clobber=clobber)
    return outputters.RegressionOutput(outim, outputters.output_resid)


def generate_output(outputs, iterable, reshape=lambda x, y: (x, y)):
    """
    Write out results of a given output.

    In the regression setting, results is generally going to be a
    scipy.stats.models.model.LikelihoodModelResults instance.

    Parameters
    ----------
    outputs : sequence
        sequence of output objects
    iterable : object
        Object which iterates, returning tuples of (indexer, results), where
        ``indexer`` can be used to index into the `outputs`
    reshape : callable
        accepts two arguments, first is the indexer, and the second is the array
        which will be indexed; returns modified indexer and array ready for
        slicing with modified indexer.
    """
    for indexer, results in iterable:
        for output in outputs:
            # Might be regression output object
            if not hasattr(output, "list"): # lame test here
                k, d = reshape(indexer, output(results))
                output[k] = d
            else:
                # or a regression output list (like a TOutput, with several
                # images to output to)
                r = output(results)
                for j, l in enumerate(output.list):
                    k, d = reshape(indexer, r[j])
                    l[k] = d
    # flush outputs, if necessary
    for output in outputs:
        if isinstance(output, outputters.RegressionOutput):
            if hasattr(output.img, 'save'):
                output.img.save()
        elif isinstance(output, outputters.RegressionOutputList):
            for im in output.list:
                if hasattr(im, 'save'):
                    im.save()



#!/usr/bin/env python

import numpy as np

from ....core.image.image_spaces import make_xyz_image
from ..affine import Affine
from ..histogram_registration import HistogramRegistration
from .._registration import _joint_histogram

from numpy.testing import assert_array_equal
from ....testing import assert_equal, assert_almost_equal, assert_raises

dummy_affine = np.eye(4)

def make_data_bool(dx=100, dy=100, dz=50):
    return (np.random.rand(dx, dy, dz)
                   - np.random.rand()) > 0


def make_data_uint8(dx=100, dy=100, dz=50):
    return (256 * (np.random.rand(dx, dy, dz)
                   - np.random.rand())).astype('uint8')


def make_data_int16(dx=100, dy=100, dz=50):
    return (256 * (np.random.rand(dx, dy, dz)
                   - np.random.rand())).astype('int16')


def make_data_float64(dx=100, dy=100, dz=50):
    return (256 * (np.random.rand(dx, dy, dz)
                 - np.random.rand())).astype('float64')


def _test_clamping(I, thI=0.0, clI=256, mask=None):
    R = HistogramRegistration(I, I, from_bins=clI, from_mask=mask, to_mask=mask)
    R.subsample(spacing=[1, 1, 1])
    Ic = R._from_data
    Ic2 = R._to_data[1:-1, 1:-1, 1:-1]
    assert_equal(Ic, Ic2)
    dyn = Ic.max() + 1
    assert_equal(dyn, R._joint_hist.shape[0])
    assert_equal(dyn, R._joint_hist.shape[1])
    return Ic, Ic2


def test_clamping_uint8():
    I = make_xyz_image(make_data_uint8(), dummy_affine, 'scanner')
    _test_clamping(I)


def test_clamping_uint8_nonstd():
    I = make_xyz_image(make_data_uint8(), dummy_affine, 'scanner')
    _test_clamping(I, 10, 165)


def test_clamping_int16():
    I = make_xyz_image(make_data_int16(), dummy_affine, 'scanner')
    _test_clamping(I)


def test_masked_clamping_int16():
    I = make_xyz_image(make_data_int16(), dummy_affine, 'scanner')
    _test_clamping(I, mask=make_data_bool())


def test_clamping_int16_nonstd():
    I = make_xyz_image(make_data_int16(), dummy_affine, 'scanner')
    _test_clamping(I, 10, 165)


def test_clamping_float64():
    I = make_xyz_image(make_data_float64(), dummy_affine, 'scanner')
    _test_clamping(I)


def test_clamping_float64_nonstd():
    I = make_xyz_image(make_data_float64(), dummy_affine, 'scanner')
    _test_clamping(I, 10, 165)


def _test_similarity_measure(simi, val):
    I = make_xyz_image(make_data_int16(), dummy_affine, 'scanner')
    J = make_xyz_image(I.get_data().copy(), dummy_affine, 'scanner')
    R = HistogramRegistration(I, J)
    R.subsample(spacing=[2, 1, 3])
    R.similarity = simi
    assert_almost_equal(R.eval(Affine()), val)


def test_correlation_coefficient():
    _test_similarity_measure('cc', 1.0)


def test_correlation_ratio():
    _test_similarity_measure('cr', 1.0)


def test_correlation_ratio_L1():
    _test_similarity_measure('crl1', 1.0)


def test_normalized_mutual_information():
    _test_similarity_measure('nmi', 1.0)


def test_joint_hist_eval():
    I = make_xyz_image(make_data_int16(), dummy_affine, 'scanner')
    J = make_xyz_image(I.get_data().copy(), dummy_affine, 'scanner')
    # Obviously the data should be the same
    assert_array_equal(I.get_data(), J.get_data())
    # Instantiate default thing
    R = HistogramRegistration(I, J)
    R.similarity = 'cc'
    null_affine = Affine()
    val = R.eval(null_affine)
    assert_almost_equal(val, 1.0)
    # Try with what should be identity
    R.subsample(spacing=[1, 1, 1])
    assert_array_equal(R._from_data.shape, I.shape)
    val = R.eval(null_affine)
    assert_almost_equal(val, 1.0)


def test_joint_hist_raw():
    # Set up call to joint histogram
    jh_arr = np.zeros((10, 10), dtype=np.double)
    data_shape = (2, 3, 4)
    data = np.random.randint(size=data_shape,
                             low=0, high=10).astype(np.short)
    data2 = np.zeros(np.array(data_shape) + 2, dtype=np.short)
    data2[:] = -1
    data2[1:-1, 1:-1, 1:-1] = data.copy()
    vox_coords = np.indices(data_shape).transpose((1, 2, 3, 0))
    vox_coords = np.ascontiguousarray(vox_coords.astype(np.double))
    _joint_histogram(jh_arr, data.flat, data2, vox_coords, 0)
    assert_almost_equal(np.diag(np.diag(jh_arr)), jh_arr)


def test_explore():
    I = make_xyz_image(make_data_int16(), dummy_affine, 'scanner')
    J = make_xyz_image(make_data_int16(), dummy_affine, 'scanner')
    R = HistogramRegistration(I, J)
    T = Affine()
    simi, params = R.explore(T, (0, [-1, 0, 1]), (1, [-1, 0, 1]))


def test_histogram_registration():
    """ Test the histogram registration class.
    """
    I = make_xyz_image(make_data_int16(), dummy_affine, 'scanner')
    J = make_xyz_image(I.get_data().copy(), dummy_affine, 'scanner')
    R = HistogramRegistration(I, J)
    assert_raises(ValueError, R.subsample, spacing=[0, 1, 3])


def test_set_fov():
    I = make_xyz_image(make_data_int16(), dummy_affine, 'scanner')
    J = make_xyz_image(I.get_data().copy(), dummy_affine, 'scanner')
    R = HistogramRegistration(I, J)
    R.set_fov(npoints=np.prod(I.shape))
    assert_equal(R._from_data.shape, I.shape)
    half_shape = tuple([I.shape[i] / 2 for i in range(3)])
    R.set_fov(spacing=(2, 2, 2))
    assert_equal(R._from_data.shape, half_shape)
    R.set_fov(corner=half_shape)
    assert_equal(R._from_data.shape, half_shape)
    R.set_fov(size=half_shape)
    assert_equal(R._from_data.shape, half_shape)


def test_histogram_masked_registration():
    """ Test the histogram registration class.
    """
    I = make_xyz_image(make_data_int16(dx=100, dy=100, dz=50), dummy_affine, 'scanner')
    J = make_xyz_image(make_data_int16(dx=100, dy=100, dz=50), dummy_affine, 'scanner')
    mask = (np.zeros((100,100,50)) == 1)
    mask[10:20,10:20,10:20] = True
    R = HistogramRegistration(I, J, to_mask=mask, from_mask=mask)
    sim1 = R.eval(Affine())
    I = make_xyz_image(I.get_data()[mask].reshape(10,10,10), dummy_affine, 'scanner')
    J = make_xyz_image(J.get_data()[mask].reshape(10,10,10), dummy_affine, 'scanner')
    R = HistogramRegistration(I, J)
    sim2 = R.eval(Affine())
    assert_equal(sim1, sim2)


if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np

from ..spatial_relaxation_onesample import multivariate_stat

from ....testing import (dec, assert_true, assert_equal, assert_almost_equal)

verbose = False

def make_data(n=10, dim=np.array([20,20,20]), r=5, amplitude=10, noise=1,
              jitter=None, prng=np.random):
    if np.isscalar(dim):
        dim = np.array([dim,dim,dim])
    XYZvol = np.zeros((dim),int)
    XYZ = np.array(np.where(XYZvol==0))
    p = XYZ.shape[1]
    #mask = np.arange(p)
    XYZvol[XYZ[0],XYZ[1],XYZ[2]] = np.arange(p)
    o = np.array(dim/2)
    maskvol = np.zeros(dim, int)
    maskvol[list(XYZ)] = np.arange(p)
    Isignal = maskvol[dim[0]/2-r:dim[0]/2+r, dim[1]/2-r:dim[1]/2+r, dim[2]/2-r:dim[2]/2+r].ravel()
    signal = np.zeros(p, float)
    signal[Isignal] += amplitude
    X = np.zeros((n, p), float) + np.nan
    data = np.zeros((n, p), float) + np.nan
    vardata = np.zeros((n, p), float) + np.nan
    for i in xrange(n):
        X[i] = prng.randn(p)
        o = np.array(dim/2)
        if jitter!=None:
            # Not in place to avoid stricter numpy 2 casting
            o = o + np.round(prng.randn(3)*jitter).clip(r-dim/2,dim/2-r)
        Ii = XYZvol[o[0]-r:o[0]+r, o[1]-r:o[1]+r, o[2]-r:o[2]+r].ravel()
        X[i,Ii] += amplitude
        vardata[i] = np.square(prng.randn(p))*noise**2
        data[i] = X[i] + prng.randn(p)*np.sqrt(vardata[i])
    return data, XYZ, XYZvol, vardata, signal


def test_evaluate_exact():
    # without mfx nor spatial relaxation
    prng = np.random.RandomState(10)
    data, XYZ, XYZvol, vardata, signal = make_data(n=20,
            dim=np.array([20,20,20]), r=3, amplitude=5, noise=0,
            jitter=0, prng=prng)
    p = len(signal)
    XYZvol *= 0
    XYZvol[list(XYZ)] = np.arange(p)
    P = multivariate_stat(data)
    P.init_hidden_variables()
    P.evaluate(nsimu=100, burnin=100, J=[XYZvol[5, 5, 5]],
               compute_post_mean=True, verbose=verbose)
    P.log_likelihood_values = P.compute_log_region_likelihood()
    # Verify code consistency
    Q = multivariate_stat(data, vardata*0, XYZ, std=0, sigma=5)
    Q.init_hidden_variables()
    Q.evaluate(nsimu=100, burnin=100, J = [XYZvol[5,5,5]],
               compute_post_mean=True, update_spatial=False,
               verbose=verbose)
    Q.log_likelihood_values = Q.compute_log_region_likelihood()
    assert_almost_equal(P.mean_m.mean(),
                              Q.mean_m.mean(),
                              int(np.log10(P.nsimu))-1)
    assert_almost_equal(Q.log_likelihood_values.sum(),
                              P.log_likelihood_values.sum(), 0)


def test_model_selection_exact():
    prng = np.random.RandomState(10)
    data, XYZ, XYZvol, vardata, signal = make_data(n=30, dim=20, r=3,
                amplitude=1, noise=0, jitter=0, prng=prng)
    labels = (signal > 0).astype(int)
    P1 = multivariate_stat(data, labels=labels)
    P1.init_hidden_variables()
    P1.evaluate(nsimu=100, burnin=10, verbose=verbose)
    L1 = P1.compute_log_region_likelihood()
    Prior1 = P1.compute_log_prior()
    #v, m_mean, m_var = P1.v.copy(), P1.m_mean.copy(), P1.m_var.copy()
    Post1 = P1.compute_log_posterior(nsimu=1e2, burnin=1e2, verbose=verbose)
    M1 = L1 + Prior1[:-1] - Post1[:-1]
    assert_almost_equal(M1.mean(), P1.compute_marginal_likelihood().mean(), 0)
    P0 = multivariate_stat(data, labels=labels)
    P0.network *= 0
    P0.init_hidden_variables()
    P0.evaluate(nsimu=100, burnin=100, verbose=verbose)
    L0 = P0.compute_log_region_likelihood()
    Prior0 = P0.compute_log_prior()
    Post0 = P0.compute_log_posterior(nsimu=1e2, burnin=1e2,
                                     verbose=verbose)
    M0 = L0 + Prior0[:-1] - Post0[:-1]
    assert_almost_equal(M0.mean(), P0.compute_marginal_likelihood().mean(), 0)
    assert_true(M1[1] > M0[1])
    assert_true(M1[0] < M0[0])


@dec.slow # test takes around 7 minutes on my (MB) Mac laptop
def test_model_selection_mfx_spatial_rand_walk():
    prng = np.random.RandomState(10)
    data, XYZ, XYZvol, vardata, signal = make_data(
        n=20,
        dim=np.array([1,20,20]),
        r=3, amplitude=3, noise=1,
        jitter=0.5, prng=prng)
    labels = (signal > 0).astype(int)
    P = multivariate_stat(data, vardata, XYZ, std=0.5, sigma=5, labels=labels)
    P.network[:] = 0
    P.init_hidden_variables()
    P.evaluate(nsimu=100, burnin=100, verbose=verbose,
               proposal='rand_walk', proposal_std=0.5)
    L00 = P.compute_log_region_likelihood()
    # Test simulated annealing procedure
    P.estimate_displacements_SA(nsimu=100, c=0.99,
                                proposal_std=P.proposal_std, verbose=verbose)
    L0 = P.compute_log_region_likelihood()
    assert_true(L0.sum() > L00.sum())
    #Prior0 = P.compute_log_prior()
    #Post0 = P.compute_log_posterior(nsimu=1e2, burnin=1e2, verbose=verbose)
    #M0 = L0 + Prior0[:-1] - Post0[:-1]
    M0 = P.compute_marginal_likelihood(update_spatial=True)
    #yield assert_almost_equal(M0.sum(), P.compute_marginal_likelihood(verbose=verbose).sum(), 0)
    P.network[1] = 1
    #P.init_hidden_variables(init_spatial=False)
    P.init_hidden_variables(init_spatial=False)
    P.evaluate(nsimu=100, burnin=100, verbose=verbose,
               update_spatial=False, proposal_std=P.proposal_std)
    #L1 = P.compute_log_region_likelihood()
    #Prior1 = P.compute_log_prior()
    #Post1 = P.compute_log_posterior(nsimu=1e2, burnin=1e2, verbose=verbose)
    #M1 = L1 + Prior1[:-1] - Post1[:-1]
    M1 = P.compute_marginal_likelihood(update_spatial=True)
    #yield assert_almost_equal(0.1*M1.sum(), 0.1*P.compute_marginal_likelihood(verbose=verbose).sum(), 0)
    assert_true(M1 > M0)


def test_evaluate_mfx_spatial():
    prng = np.random.RandomState(10)
    data, XYZ, XYZvol, vardata, signal = make_data(
        n=20,
        dim=10, r=3, amplitude=5, noise=1, jitter=1,
        prng=prng)
    P = multivariate_stat(data, vardata, XYZ, std=1, sigma=3)
    P.init_hidden_variables()
    P.evaluate(nsimu=5, burnin=5,
               J=[P.D.XYZ_vol[10, 10, 10]],
               verbose=verbose, mode='mcmc')
    # Test log_likelihood computation
    v = P.v.copy()
    m_var = P.m_var.copy()
    m_mean = P.m_mean.copy()
    L1 = P.compute_log_region_likelihood_slow(v, m_mean, m_var)
    L2 = P.compute_log_region_likelihood(v, m_mean, m_var)
    assert_almost_equal(-L1.sum(), L2.sum()*2, 2)
    # Test posterior density computation
    #Prior = P.compute_log_prior(v, m_mean, m_var)
    #Post = P.compute_log_posterior(v, m_mean, m_var, nsimu=10, 
                      #burnin=10, verbose=verbose)


def test_update_labels():
    prng = np.random.RandomState(10)
    data, XYZ, XYZvol, vardata, signal = make_data(
        n=20, dim=20, r=3,
        amplitude=5, noise=1, jitter=1, prng=prng)
    P = multivariate_stat(data, vardata, XYZ)
    P.init_hidden_variables()
    p = P.data.shape[1]
    P.labels_prior = np.ones((1, p), float)
    P.label_values = np.zeros((1, p), int)
    P.labels_prior_mask = np.arange(p)
    P.update_labels()
    assert_equal(max(abs(P.labels - np.zeros(p, int))), 0)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np
from ._segmentation import _ve_step, _interaction_energy

NITERS = 10
NGB_SIZE = 26
BETA = 0.1

nonzero = lambda x: np.maximum(x, 1e-50)
log = lambda x: np.log(nonzero(x))


class Segmentation(object):

    def __init__(self, data, mask=None, mu=None, sigma=None,
                 ppm=None, prior=None, U=None,
                 ngb_size=NGB_SIZE, beta=BETA):
        """
        Class for multichannel Markov random field image segmentation
        using the variational EM algorithm. For details regarding the
        underlying algorithm, see:

        Roche et al, 2011. On the convergence of EM-like algorithms
        for image segmentation using Markov random fields. Medical
        Image Analysis (DOI: 10.1016/j.media.2011.05.002).

        Parameters
        ----------
        data : array-like
          Input image array

        mask : array-like or tuple of array
          Input mask to restrict the segmentation

        beta : float
          Markov regularization parameter

        mu : array-like
          Initial class-specific means

        sigma : array-like
          Initial class-specific variances
        """
        data = data.squeeze()
        if not len(data.shape) in (3, 4):
            raise ValueError('Invalid input image')
        if len(data.shape) == 3:
            nchannels = 1
            space_shape = data.shape
        else:
            nchannels = data.shape[-1]
            space_shape = data.shape[0:-1]

        self.nchannels = nchannels

        # Make default mask (required by MRF regularization). This wil
        # be passed to the _ve_step C-routine, which assumes a
        # contiguous int array and raise an error otherwise. Voxels on
        # the image borders are further rejected to avoid segmentation
        # faults.
        if mask == None:
            mask = np.ones(space_shape, dtype=bool)
        X, Y, Z = np.where(mask)
        XYZ = np.zeros((X.shape[0], 3), dtype='intp')
        XYZ[:, 0], XYZ[:, 1], XYZ[:, 2] = X, Y, Z
        self.XYZ = XYZ

        self.mask = mask
        self.data = data[mask]
        if nchannels == 1:
            self.data = np.reshape(self.data, (self.data.shape[0], 1))

        # By default, the ppm is initialized as a collection of
        # uniform distributions
        if ppm == None:
            nclasses = len(mu)
            self.ppm = np.zeros(list(space_shape) + [nclasses])
            self.ppm[mask] = 1. / nclasses
            self.is_ppm = False
            self.mu = np.array(mu, dtype='double').reshape(\
                (nclasses, nchannels))
            self.sigma = np.array(sigma, dtype='double').reshape(\
                (nclasses, nchannels, nchannels))
        elif mu == None:
            nclasses = ppm.shape[-1]
            self.ppm = np.asarray(ppm)
            self.is_ppm = True
            self.mu = np.zeros((nclasses, nchannels))
            self.sigma = np.zeros((nclasses, nchannels, nchannels))
        else:
            raise ValueError('missing information')
        self.nclasses = nclasses

        if not prior == None:
            self.prior = np.asarray(prior)[self.mask].reshape(\
                [self.data.shape[0], nclasses])
        else:
            self.prior = None

        self.ngb_size = int(ngb_size)
        self.set_markov_prior(beta, U=U)

    def set_markov_prior(self, beta, U=None):
        if not U == None:  # make sure it's C-contiguous
            self.U = np.asarray(U).copy()
        else:  # Potts model
            U = np.ones((self.nclasses, self.nclasses))
            U[_diag_indices(self.nclasses)] = 0
            self.U = U
        self.beta = float(beta)

    def vm_step(self, freeze=()):
        classes = range(self.nclasses)
        for i in freeze:
            classes.remove(i)

        for i in classes:
            P = self.ppm[..., i][self.mask].ravel()
            Z = nonzero(P.sum())
            tmp = self.data.T * P.T
            mu = tmp.sum(1) / Z
            mu_ = mu.reshape((len(mu), 1))
            sigma = np.dot(tmp, self.data) / Z - np.dot(mu_, mu_.T)
            self.mu[i] = mu
            self.sigma[i] = sigma

    def log_external_field(self):
        """
        Compute the logarithm of the external field, where the
        external field is defined as the likelihood times the
        first-order component of the prior.
        """
        lef = np.zeros([self.data.shape[0], self.nclasses])

        for i in range(self.nclasses):
            centered_data = self.data - self.mu[i]
            if self.nchannels == 1:
                inv_sigma = 1. / nonzero(self.sigma[i])
                norm_factor = np.sqrt(inv_sigma.squeeze())
            else:
                inv_sigma = np.linalg.inv(self.sigma[i])
                norm_factor = 1. / np.sqrt(\
                    nonzero(np.linalg.det(self.sigma[i])))
            maha_dist = np.sum(centered_data * np.dot(inv_sigma,
                                                      centered_data.T).T, 1)
            lef[:, i] = -.5 * maha_dist
            lef[:, i] += log(norm_factor)

        if not self.prior == None:
            lef += log(self.prior)

        return lef

    def normalized_external_field(self):
        f = self.log_external_field().T
        f -= np.max(f, 0)
        np.exp(f, f)
        f /= f.sum(0)
        return f.T

    def ve_step(self):
        nef = self.normalized_external_field()
        if self.beta == 0:
            self.ppm[self.mask] = np.reshape(\
                nef, self.ppm[self.mask].shape)
        else:
            self.ppm = _ve_step(self.ppm, nef, self.XYZ,
                                self.U, self.ngb_size, self.beta)

    def run(self, niters=NITERS, freeze=()):
        if self.is_ppm:
            self.vm_step(freeze=freeze)
        for i in range(niters):
            self.ve_step()
            self.vm_step(freeze=freeze)
        self.is_ppm = True

    def map(self):
        """
        Return the maximum a posterior label map
        """
        return map_from_ppm(self.ppm, self.mask)

    def free_energy(self, ppm=None):
        """
        Compute the free energy defined as:

        F(q, theta) = int q(x) log q(x)/p(x,y/theta) dx

        associated with input parameters mu,
        sigma and beta (up to an ignored constant).
        """
        if ppm == None:
            ppm = self.ppm
        q = ppm[self.mask]
        # Entropy term
        lef = self.log_external_field()
        f1 = np.sum(q * (log(q) - lef))
        # Interaction term
        if self.beta > 0.0:
            f2 = self.beta * _interaction_energy(ppm, self.XYZ,
                                                 self.U, self.ngb_size)
        else:
            f2 = 0.0
        return f1 + f2


def _diag_indices(n, ndim=2):
    # diag_indices function present in numpy 1.4 and later.  This for
    # compatibility with numpy < 1.4
    idx = np.arange(n)
    return (idx,) * ndim


def moment_matching(dat, mu, sigma, glob_mu, glob_sigma):
    """
    Moment matching strategy for parameter initialization to feed a
    segmentation algorithm.

    Parameters
    ----------
    data: array
      Image data.

    mu : array
      Template class-specific intensity means

    sigma : array
      Template class-specific intensity variances

    glob_mu : float
      Template global intensity mean

    glob_sigma : float
      Template global intensity variance

    Returns
    -------
    dat_mu: array
      Guess of class-specific intensity means

    dat_sigma: array
      Guess of class-specific intensity variances
    """
    dat_glob_mu = float(np.mean(dat))
    dat_glob_sigma = float(np.var(dat))
    a = np.sqrt(dat_glob_sigma / glob_sigma)
    b = dat_glob_mu - a * glob_mu
    dat_mu = a * mu + b
    dat_sigma = (a ** 2) * sigma
    return dat_mu, dat_sigma


def map_from_ppm(ppm, mask=None):
    x = np.zeros(ppm.shape[0:-1], dtype='uint8')
    if mask == None:
        mask = ppm == 0
    x[mask] = ppm[mask].argmax(-1) + 1
    return x


def binarize_ppm(q):
    """
    Assume input ppm is masked (ndim==2)
    """
    bin_q = np.zeros(q.shape)
    bin_q[(range(q.shape[0]), np.argmax(q, axis=1))] = 1.
    return bin_q



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Test surrogate data generation.
"""

import numpy as np
from nose.tools import assert_true
from nibabel import Nifti1Image

from ..simul_multisubject_fmri_dataset import \
    surrogate_2d_dataset, surrogate_3d_dataset, surrogate_4d_dataset 


def test_surrogate_array():
    """ Check that with no noise, the surrogate activation correspond to
        the ones that we specify. 2D version
    """
    # We can't use random positions, as the positions have to be
    # far-enough not to overlap.
    pos   = np.array([[ 2, 10],
                      [10,  4],
                      [19, 15],
                      [15, 19],
                      [5, 18]])
    ampli = np.random.random(5)
    data = surrogate_2d_dataset(n_subj=1, noise_level=0, spatial_jitter=0,
                                signal_jitter=0, pos=pos, shape=(20,20),
                                ampli=ampli).squeeze()
    x, y = pos.T
    np.testing.assert_array_equal(data[x, y], ampli)


def test_surrogate_array_3d():
    """ Check that with no noise, the surrogate activation correspond to
        the ones that we specify. 3D version
    """
    # We can't use random positions, as the positions have to be
    # far-enough not to overlap.
    pos   = np.array([[ 2, 10, 2],
                      [10,  4, 4],
                      [18, 13, 18],
                      [13, 18, 5],
                      [5, 18, 18]])
    ampli = np.random.random(5)
    data = surrogate_3d_dataset(n_subj=1, noise_level=0, spatial_jitter=0,
                                signal_jitter=0, pos=pos, shape=(20,20,20),
                                ampli=ampli).squeeze()
    x, y, z = pos.T
    np.testing.assert_array_equal(data[x, y, z], ampli)


def test_surrogate_array_3d_write():
    """ Check that 3D version spits files when required
    """
    from os import path
    from tempfile import mkdtemp
    write_path = path.join(mkdtemp(), 'img.nii') 
    shape = (5, 6, 7)
    data = surrogate_3d_dataset(shape=shape, out_image_file=write_path)
    assert_true(path.isfile(write_path))

def test_surrogate_array_3d_mask():
    """ Check that 3D version works when a mask is provided
    """
    shape = (5, 6, 7)
    mask = np.random.rand(*shape) > 0.5
    mask_img = Nifti1Image(mask.astype(np.uint8), np.eye(4))
    img = surrogate_3d_dataset(mask=mask_img)
    mean_image  = img[mask].mean()
    assert_true((img[mask == 0] == 0).all())


def test_surrogate_array_4d_shape():
    """Run the 4D datageneration; check the output shape and length
    """
    shape = (5, 6, 7)
    out_shape = shape + (1,)
    imgs = surrogate_4d_dataset(shape)
    assert_true(not np.any(np.asarray(imgs[0].shape) - np.asarray(out_shape)))
    n_sess = 3
    imgs = surrogate_4d_dataset(shape, n_sess=n_sess)
    assert_true(imgs[0].shape == out_shape)
    assert_true(len(imgs) == n_sess)
    n_scans = 5
    out_shape = shape + (n_scans,)
    imgs = surrogate_4d_dataset(shape, n_scans=n_scans)
    assert_true(imgs[0].shape == (out_shape))


def test_surrogate_array_4d_write():
    """Run the 4D data generation; check that output images are written
    """
    from os import path
    from tempfile import mkdtemp
    n_sess = 3
    write_paths = [path.join(mkdtemp(), 'img_%d.nii' % i) 
                   for i in range(n_sess)]
    shape = (5, 6, 7)
    imgs = surrogate_4d_dataset(shape, out_image_file=write_paths[0])
    assert_true(path.isfile(write_paths[0]))
    imgs = surrogate_4d_dataset(shape, n_sess=n_sess, 
                                out_image_file=write_paths)
    for wp in write_paths:
        assert_true(path.isfile(wp))


def test_surrogate_array_4d_mask():
    """Run the 4D version, with masking
    """
    shape = (5, 5, 5)
    mask = np.random.rand(*shape) > 0.5
    mask_img = Nifti1Image(mask.astype(np.uint8), np.eye(4))
    imgs = surrogate_4d_dataset(mask=mask_img)
    mean_image  = imgs[0].get_data()[mask].mean()
    assert_true((imgs[0].get_data()[mask == 0] < mean_image / 2).all())


def test_surrogate_array_4d_dmtx():
    """Run the 4D version, with design_matrix provided
    """
    shape = (5, 5, 5)
    n_scans = 25
    out_shape = shape + (n_scans,)
    dmtx = np.random.randn(n_scans, 3)
    imgs = surrogate_4d_dataset(shape, dmtx=dmtx)
    assert_true(not np.any(np.asarray(imgs[0].shape) - np.asarray(out_shape)))


if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])



""" This file contains defines parameters for nipy that we use to fill
settings in setup.py, the nipy top-level docstring, and for building the
docs.  In setup.py in particular, we exec this file, so it cannot import nipy
"""

# nipy version information.  An empty _version_extra corresponds to a
# full release.  '.dev' as a _version_extra string means this is a development
# version
_version_major = 0
_version_minor = 3
_version_micro = 0
#_version_extra = '.dev' # For development
_version_extra = '' # For release

# Format expected by setup.py and doc/source/conf.py: string of form "X.Y.Z"
__version__ = "%s.%s.%s%s" % (_version_major,
                              _version_minor,
                              _version_micro,
                              _version_extra)

CLASSIFIERS = ["Development Status :: 3 - Alpha",
               "Environment :: Console",
               "Intended Audience :: Science/Research",
               "License :: OSI Approved :: BSD License",
               "Operating System :: OS Independent",
               "Programming Language :: Python",
               "Topic :: Scientific/Engineering"]

description  = 'A python package for analysis of neuroimaging data'

# Note: this long_description is actually a copy/paste from the top-level
# README.rst, so that it shows up nicely on PyPI.  So please remember to edit
# it only in one place and sync it correctly.
long_description = \
"""
====
NIPY
====

Neuroimaging tools for Python.

The aim of NIPY is to produce a platform-independent Python environment for the
analysis of functional brain imaging data using an open development model.

In NIPY we aim to:

1. Provide an open source, mixed language scientific programming
    environment suitable for rapid development.

2. Create sofware components in this environment to make it easy
    to develop tools for MRI, EEG, PET and other modalities.

3. Create and maintain a wide base of developers to contribute to
    this platform.

4. To maintain and develop this framework as a single, easily
    installable bundle.

NIPY is the work of many people. We list the main authors in the file ``AUTHOR``
in the NIPY distribution, and other contributions in ``THANKS``.

Website
=======

Current information can always be found at the NIPY website::

    http://nipy.org/nipy

Mailing Lists
=============

Please see the developer's list::

    http://projects.scipy.org/mailman/listinfo/nipy-devel

Code
====

You can find our sources and single-click downloads:

* `Main repository`_ on Github.
* Documentation_ for all releases and current development tree.
* Download as a tar/zip file the `current trunk`_.
* Downloads of all `available releases`_.

.. _main repository: http://github.com/nipy/nipy
.. _Documentation: http://nipy.org/nipy
.. _current trunk: http://github.com/nipy/nipy/archives/master
.. _available releases: http://pypi.python.org/pypi/nipy

Dependencies
============

To run NIPY, you will need:

* python_ >= 2.5 (tested with 2.5, 2.6, 2.7, 3.2, 3.3)
* numpy_ >= 1.2
* scipy_ >= 0.7.0
* sympy_ >= 0.6.6
* nibabel_ >= 1.2

You will probably also like to have:

* ipython_ for interactive work
* matplotlib_ for 2D plotting
* mayavi_ for 3D plotting

.. _python: http://python.org
.. _numpy: http://numpy.scipy.org
.. _scipy: http://www.scipy.org
.. _sympy: http://sympy.org
.. _nibabel: http://nipy.org/nibabel
.. _ipython: http://ipython.scipy.org
.. _matplotlib: http://matplotlib.sourceforge.net
.. _mayavi: http://code.enthought.com/projects/mayavi/

License
=======

We use the 3-clause BSD license; the full license is in the file ``LICENSE`` in
the nipy distribution.
"""

NAME                = 'nipy'
MAINTAINER          = "nipy developers"
MAINTAINER_EMAIL    = "nipy-devel@neuroimaging.scipy.org"
DESCRIPTION         = description
LONG_DESCRIPTION    = long_description
URL                 = "http://nipy.org/nipy"
DOWNLOAD_URL        = "http://github.com/nipy/nipy/archives/master"
LICENSE             = "BSD license"
CLASSIFIERS         = CLASSIFIERS
AUTHOR              = "nipy developmers"
AUTHOR_EMAIL        = "nipy-devel@neuroimaging.scipy.org"
PLATFORMS           = "OS Independent"
MAJOR               = _version_major
MINOR               = _version_minor
MICRO               = _version_micro
ISRELEASE           = _version_extra == ''
VERSION             = __version__
REQUIRES            = ["numpy", "scipy", "sympy"]
STATUS              = 'beta'

# versions
NUMPY_MIN_VERSION='1.2'
SCIPY_MIN_VERSION = '0.7'
NIBABEL_MIN_VERSION = '1.2'
SYMPY_MIN_VERSION = '0.6.6'
MAYAVI_MIN_VERSION = '3.0'
CYTHON_MIN_VERSION = '0.12.1'

# Versions and locations of optional data packages
NIPY_DATA_URL= 'http://nipy.sourceforge.net/data-packages/'
DATA_PKGS = {'nipy-data': {'min version':'0.2',
                           'relpath':'nipy/data'},
             'nipy-templates': {'min version':'0.2',
                                'relpath':'nipy/templates'}
            }
NIPY_INSTALL_HINT = \
"""You can download and install the package from:

%s

Check the instructions in the ``doc/users/install_data.rst`` file in the nipy
source tree, or online at http://nipy.org/nipy/stable/users/install_data.html

If you have the package, have you set the path to the package correctly?"""

for key, value in DATA_PKGS.items():
    url = '%s%s-%s.tar.gz' % (NIPY_DATA_URL,
                              key,
                              value['min version'])
    value['name'] = key
    value['install hint'] = NIPY_INSTALL_HINT % url

del key, value, url




# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" Test for smoothing with kernels """
import numpy as np
from numpy.random import random_integers as randint

from ... import load_image
from ..kernel_smooth import LinearFilter, sigma2fwhm, fwhm2sigma
from ...externals.transforms3d.taitbryan import euler2mat
from ...core.api import Image, compose, AffineTransform, drop_io_dim

from nose.tools import (assert_true, assert_false, assert_equal, assert_raises)

from numpy.testing import assert_array_equal, assert_array_almost_equal

from ...testing import (anatfile, funcfile)


def test_anat_smooth():
    anat = load_image(anatfile)
    smoother = LinearFilter(anat.coordmap, anat.shape)
    sanat = smoother.smooth(anat)
    assert_equal(sanat.shape, anat.shape)
    assert_equal(sanat.coordmap, anat.coordmap)
    assert_false(np.allclose(sanat.get_data(), anat.get_data()))


def test_funny_coordmap():
    # 5x4 affine should also work, and give same answer as 4x4
    func = load_image(funcfile)
    cmap = func.coordmap
    # Give the affine a rotation
    aff = np.eye(5)
    aff[:3,:3] = euler2mat(0.3, 0.2, 0.1)
    cmap_rot = AffineTransform(cmap.function_range,
                               cmap.function_range,
                               aff)
    func_rot = Image(func.get_data(), compose(cmap_rot, cmap))
    func1 = func_rot[...,1] # 5x4 affine
    smoother = LinearFilter(func1.coordmap, func1.shape)
    sfunc1 = smoother.smooth(func1) # OK
    # And same as for 4x4 affine
    cmap3d = drop_io_dim(cmap, 't')
    func3d = Image(func1.get_data(), cmap3d)
    smoother = LinearFilter(func3d.coordmap, func3d.shape)
    sfunc3d = smoother.smooth(func3d)
    assert_equal(sfunc1.shape, sfunc3d.shape)
    assert_array_almost_equal(sfunc1.get_data(), sfunc3d.get_data())
    # And same with no rotation
    func_fresh = func[...,1] # 5x4 affine, no rotation
    smoother = LinearFilter(func_fresh.coordmap, func_fresh.shape)
    sfunc_fresh = smoother.smooth(func_fresh)
    assert_equal(sfunc1.shape, sfunc_fresh.shape)
    assert_array_almost_equal(sfunc1.get_data(), sfunc_fresh.get_data())


def test_func_smooth():
    func = load_image(funcfile)
    smoother = LinearFilter(func.coordmap, func.shape)
    # should work, but currently broken : sfunc = smoother.smooth(func)
    assert_raises(NotImplementedError, smoother.smooth, func)


def test_sigma_fwhm():
    # ensure that fwhm2sigma and sigma2fwhm are inverses of each other
    fwhm = np.arange(1.0, 5.0, 0.1)
    sigma = np.arange(1.0, 5.0, 0.1)
    assert_true(np.allclose(sigma2fwhm(fwhm2sigma(fwhm)), fwhm))
    assert_true(np.allclose(fwhm2sigma(sigma2fwhm(sigma)), sigma))


def test_kernel():
    # Verify that convolution with a delta function gives the correct
    # answer.
    tol = 0.9999
    sdtol = 1.0e-8
    for x in range(6):
        shape = randint(30,60,(3,))
        # pos of delta
        ii, jj, kk = randint(11,17, (3,))
        # random affine coordmap (diagonal and translations)
        coordmap = AffineTransform.from_start_step('ijk', 'xyz', 
                                          randint(5,20,(3,))*0.25,
                                          randint(5,10,(3,))*0.5)
        # delta function in 3D array
        signal = np.zeros(shape)
        signal[ii,jj,kk] = 1.
        signal = Image(signal, coordmap=coordmap)
        # A filter with coordmap, shape matched to image
        kernel = LinearFilter(coordmap, shape, 
                              fwhm=randint(50,100)/10.)
        # smoothed normalized 3D array
        ssignal = kernel.smooth(signal).get_data()
        ssignal[:] *= kernel.norms[kernel.normalization]
        # 3 points * signal.size array
        I = np.indices(ssignal.shape)
        I.shape = (kernel.coordmap.ndims[0], np.product(shape))
        # location of maximum in smoothed array
        i, j, k = I[:, np.argmax(ssignal[:].flat)]
        # same place as we put it before smoothing?
        assert_equal((i,j,k), (ii,jj,kk))
        # get physical points position relative to position of delta
        Z = kernel.coordmap(I.T) - kernel.coordmap([i,j,k])
        _k = kernel(Z)
        _k.shape = ssignal.shape
        assert_true((np.corrcoef(_k[:].flat, ssignal[:].flat)[0,1] > tol))
        assert_true(((_k[:] - ssignal[:]).std() < sdtol))

        def _indices(i,j,k,axis):
            I = np.zeros((3,20))
            I[0] += i
            I[1] += j
            I[2] += k
            I[axis] += np.arange(-10,10)
            return I.T

        vx = ssignal[i,j,(k-10):(k+10)]
        xformed_ijk = coordmap([i, j, k])
        vvx = coordmap(_indices(i,j,k,2)) - xformed_ijk
        assert_true((np.corrcoef(vx, kernel(vvx))[0,1] > tol))
        vy = ssignal[i,(j-10):(j+10),k]
        vvy = coordmap(_indices(i,j,k,1)) - xformed_ijk
        assert_true((np.corrcoef(vy, kernel(vvy))[0,1] > tol))
        vz = ssignal[(i-10):(i+10),j,k]
        vvz = coordmap(_indices(i,j,k,0)) - xformed_ijk
        assert_true((np.corrcoef(vz, kernel(vvz))[0,1] > tol))




# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
3D visualization of activation maps using Mayavi

"""

# Author: Gael Varoquaux <gael dot varoquaux at normalesup dot org>
# License: BSD

# Standard library imports
import os

# Standard scientific libraries imports (more specific imports are
# delayed, so that the part module can be used without them).
import numpy as np
from scipy import ndimage

from nibabel import load

# The sform for MNI templates
mni_sform = np.array([[-1, 0, 0,   90],
                      [ 0, 1, 0, -126],
                      [ 0, 0, 1,  -72],
                      [ 0, 0, 0,   1]])

mni_sform_inv = np.linalg.inv(mni_sform)

def find_mni_template():
    """ Try to find an MNI template on the disk.
    """
    from nipy.utils import templates, DataError
    try:
        filename = templates.get_filename(
                            'ICBM152', '1mm', 'T1_brain.nii.gz')
        if os.path.exists(filename):
            return filename
    except DataError:
        pass
    possible_paths = [
     ('', 'usr', 'share', 'fsl', 'data', 'standard', 'avg152T1_brain.nii.gz'),
     ('', 'usr', 'share', 'data', 'fsl-mni152-templates', 'avg152T1_brain.nii.gz'),
     ('', 'usr', 'local', 'share', 'fsl', 'data', 'standard', 'avg152T1_brain.nii.gz'),
            ]
    if 'FSLDIR' in os.environ:
        fsl_path = os.environ['FSLDIR'].split(os.sep)
        fsl_path.extend(('data', 'standard', 'avg152T1_brain.nii.gz'))
        possible_paths.append(fsl_path)
    for path in possible_paths:
        filename = os.sep.join((path))
        if os.path.exists(filename):
            return filename



################################################################################
# Caching of the MNI template.
################################################################################

class _AnatCache(object):
    """ Class to store the anat array in cache, to avoid reloading it
        each time.
    """
    anat        = None
    anat_sform  = None
    blurred     = None

    @classmethod
    def get_anat(cls):
        filename = find_mni_template()
        if cls.anat is None:
            if filename is None:
                raise OSError('Cannot find template file T1_brain.nii.gz '
                        'required to plot anatomy, see the nipy documentation '
                        'installaton section for how to install template files.')
            anat_im = load(filename)
            anat = anat_im.get_data()
            anat = anat.astype(np.float)
            anat_mask = ndimage.morphology.binary_fill_holes(anat > 0)
            anat = np.ma.masked_array(anat, np.logical_not(anat_mask))
            cls.anat_sform = anat_im.get_affine()
            cls.anat = anat
            cls.anat_max = anat.max()
        return cls.anat, cls.anat_sform, cls.anat_max


    @classmethod
    def get_blurred(cls):
        if cls.blurred is not None:
            return cls.blurred
        anat, _, _ = cls.get_anat()
        cls.blurred = ndimage.gaussian_filter(
                (ndimage.morphology.binary_fill_holes(
                    ndimage.gaussian_filter(
                            (anat > 4800).astype(np.float), 6)
                        > 0.5
                    )).astype(np.float),
                2).T.ravel()
        return cls.blurred






# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np

from ..coord_tools import coord_transform, find_cut_coords

def test_coord_transform_trivial():
    sform = np.eye(4)
    x = np.random.random((10,))
    y = np.random.random((10,))
    z = np.random.random((10,))

    x_, y_, z_ = coord_transform(x, y, z, sform)
    np.testing.assert_array_equal(x, x_)
    np.testing.assert_array_equal(y, y_)
    np.testing.assert_array_equal(z, z_)

    sform[:, -1] = 1
    x_, y_, z_ = coord_transform(x, y, z, sform)
    np.testing.assert_array_equal(x+1, x_)
    np.testing.assert_array_equal(y+1, y_)
    np.testing.assert_array_equal(z+1, z_)


def test_find_cut_coords():
    map = np.zeros((100, 100, 100))
    x_map, y_map, z_map = 50, 10, 40
    map[x_map-30:x_map+30, y_map-3:y_map+3, z_map-10:z_map+10] = 1
    x, y, z = find_cut_coords(map, mask=np.ones(map.shape, np.bool))
    np.testing.assert_array_equal(
                        (int(round(x)), int(round(y)), int(round(z))),
                                (x_map, y_map, z_map))





#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
"""
Example of script to analyse the reproducibility in group studies using a
bootstrap procedure.

This reproduces approximately the work described in 'Analysis of a large fMRI
cohort: Statistical and methodological issues for group analyses' Thirion B,
Pinel P, Meriaux S, Roche A, Dehaene S, Poline JB.  Neuroimage. 2007
Mar;35(1):105-20.

Needs matplotlib

Author: Bertrand Thirion, 2005-2009
"""

from os import getcwd, mkdir, path

from numpy import array

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

from nipy.labs.utils.reproducibility_measures import (
    group_reproducibility_metrics)

# Local import
from get_data_light import DATA_DIR, get_second_level_dataset

print('This analysis takes a long while, please be patient')

##############################################################################
# Set the paths, data, etc.
##############################################################################

nsubj = 12
nbeta = 29

data_dir = path.join(DATA_DIR, 'group_t_images')

mask_images = [path.join(data_dir, 'mask_subj%02d.nii' % n)
               for n in range(nsubj)]
stat_images = [path.join(data_dir, 'spmT_%04d_subj_%02d.nii' % (nbeta, n))
                 for n in range(nsubj)]
contrast_images = [path.join(data_dir, 'con_%04d_subj_%02d.nii' % (nbeta, n))
                 for n in range(nsubj)]
all_images = mask_images + stat_images + contrast_images
missing_file = array([not path.exists(m) for m in all_images]).any()

if missing_file:
    get_second_level_dataset()

# write directory
write_dir = path.join(getcwd(), 'results')
if not path.exists(write_dir):
    mkdir(write_dir)

##############################################################################
# main script
##############################################################################

ngroups = [4]
thresholds = [3.0, 4.0, 5.0]
sigma = 6.0
csize = 10
niter = 10
method = 'crfx'
verbose = 0
swap = False

voxel_results, cluster_results, peak_results = group_reproducibility_metrics(
    mask_images, contrast_images, [], thresholds, ngroups, method,
    cluster_threshold=csize, number_of_samples=niter, sigma=sigma,
    do_clusters=True, do_voxels=True, do_peaks=True, swap=swap)

kap = [k for k in voxel_results[ngroups[0]].values()]
clt = [k for k in cluster_results[ngroups[0]].values()]
pk = [k for k in peak_results[ngroups[0]].values()]

##############################################################################
# plot
##############################################################################

plt.figure()
plt.subplot(1, 3, 1)
plt.boxplot(kap)
plt.title('voxel-level reproducibility')
plt.xticks(range(1, 1 + len(thresholds)), thresholds)
plt.xlabel('threshold')
plt.subplot(1, 3, 2)
plt.boxplot(clt)
plt.title('cluster-level reproducibility')
plt.xticks(range(1, 1 + len(thresholds)), thresholds)
plt.xlabel('threshold')
plt.subplot(1, 3, 3)
plt.boxplot(clt)
plt.title('cluster-level reproducibility')
plt.xticks(range(1, 1 + len(thresholds)), thresholds)
plt.xlabel('threshold')


##############################################################################
# create an image
##############################################################################

"""
# this is commented until a new version of the code allows it
# with the adequate level of abstraction
th = 4.0
swap = False
kwargs = {'threshold':th,'csize':csize}
rmap = map_reproducibility(Functional, VarFunctional, grp_mask, ngroups,
                           method, swap, verbose, **kwargs)
wmap  = mask.astype(np.int)
wmap[mask] = rmap
wim = Nifti1Image(wmap, affine)
wim.get_header()['descrip']= 'reproducibility map at threshold %f, \
                             cluster size %d'%(th,csize)
wname = path.join(write_dir,'repro.nii')
save(wim, wname)

print('Wrote a reproducibility image in %s'%wname)
"""



from ..histogram import histogram

import numpy as np
from numpy.testing import assert_array_equal


def test_histogram():
    x = np.array([0,
                  1, 1,
                  2, 2, 2,
                  3, 3, 3, 3,
                  4, 4, 4, 4, 4],
                 dtype='uintp')
    h = histogram(x)
    assert_array_equal(h, [1, 2, 3, 4, 5])



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
__doc__ = """
This scipt generates a noisy activation image image and performs a watershed
segmentation in it.

Needs matplotlib

Author : Bertrand Thirion, 2009--2012
"""
#autoindent
print(__doc__)

import numpy as np

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")
import matplotlib as mpl

from nipy.labs.spatial_models.hroi import HROI_from_watershed
from nipy.labs.spatial_models.discrete_domain import grid_domain_from_shape
import nipy.labs.utils.simul_multisubject_fmri_dataset as simul

###############################################################################
# data simulation

shape = (60, 60)
pos = np.array([[12, 14],
                [20, 20],
                [30, 20]])
ampli = np.array([3, 4, 4])
x = simul.surrogate_2d_dataset(n_subj=1, shape=shape, pos=pos, ampli=ampli,
                               width=10.0).squeeze()

th = 2.36

# compute the field structure and perform the watershed
domain = grid_domain_from_shape(shape)
nroi = HROI_from_watershed(domain, np.ravel(x), threshold=th)
label = nroi.label

#compute the region-based signal average
bfm = np.array([np.mean(x.ravel()[label == k]) for k in range(label.max() + 1)])
bmap = np.zeros(x.size)
if label.max() > - 1:
    bmap[label > - 1] = bfm[label[label > - 1]]

label = np.reshape(label, shape)
bmap = np.reshape(bmap, shape)

###############################################################################
# plot the input image

aux1 = (0 - x.min()) / (x.max() - x.min())
aux2 = (bmap.max() - x.min()) / (x.max() - x.min())
cdict = {'red': ((0.0, 0.0, 0.7),
                 (aux1, 0.7, 0.7),
                 (aux2, 1.0, 1.0),
                 (1.0, 1.0, 1.0)),
       'green': ((0.0, 0.0, 0.7),
                 (aux1, 0.7, 0.0),
                 (aux2, 1.0, 1.0),
                 (1.0, 1.0, 1.0)),
        'blue': ((0.0, 0.0, 0.7),
                 (aux1, 0.7, 0.0),
                 (aux2, 0.5, 0.5),
                 (1.0, 1.0, 1.0))}

my_cmap = mpl.colors.LinearSegmentedColormap('my_colormap', cdict, 256)

plt.figure(figsize=(12, 3))
plt.subplot(1, 3, 1)
plt.imshow(np.squeeze(x), interpolation='nearest', cmap=my_cmap)
plt.axis('off')
plt.title('Thresholded image')

cb = plt.colorbar()
for t in cb.ax.get_yticklabels():
    t.set_fontsize(16)

###############################################################################
# plot the watershed label image
plt.subplot(1, 3, 2)
plt.imshow(label, interpolation='nearest')
plt.axis('off')
plt.colorbar()
plt.title('Labels')

###############################################################################
# plot the watershed-average image
plt.subplot(1, 3, 3)
aux = 0.01
cdict = {'red': ((0.0, 0.0, 0.7), (aux, 0.7, 0.7), (1.0, 1.0, 1.0)),
         'green': ((0.0, 0.0, 0.7), (aux, 0.7, 0.0), (1.0, 1.0, 1.0)),
         'blue': ((0.0, 0.0, 0.7), (aux, 0.7, 0.0), (1.0, 0.5, 1.0))}
my_cmap = mpl.colors.LinearSegmentedColormap('my_colormap', cdict, 256)

plt.imshow(bmap, interpolation='nearest', cmap=my_cmap)
plt.axis('off')
plt.title('Label-average')

cb = plt.colorbar()
for t in cb.ax.get_yticklabels():
    t.set_fontsize(16)

plt.show()



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np

from . import links as L
from . import varfuncs as V

class Family(object):

    """
    A class to model one-parameter exponential
    families.

    INPUTS:
       link      -- a Link instance
       variance  -- a variance function (models means as a function
                    of mean)

    """

    valid = [-np.inf, np.inf]

    tol = 1.0e-05
    links = []

    def _setlink(self, link):
        self._link = link
        if hasattr(self, "links"):
            if link not in self.links:
                raise ValueError, 'invalid link for family, should be in %s' % `self.links`

    def _getlink(self):
        return self._link

    link = property(_getlink, _setlink)

    def __init__(self, link, variance):

        self.link = link
        self.variance = variance

    def weights(self, mu):

        """
        Weights for IRLS step.

        w = 1 / (link'(mu)**2 * variance(mu))

        INPUTS:
           mu  -- mean parameter in exponential family

        OUTPUTS:
           w   -- weights used in WLS step of GLM/GAM fit

        """

        return 1. / (self.link.deriv(mu)**2 * self.variance(mu))

    def deviance(self, Y, mu, scale=1.):
        """
        Deviance of (Y,mu) pair. Deviance is usually defined
        as the difference

        DEV = (SUM_i -2 log Likelihood(Y_i,mu_i) + 2 log Likelihood(mu_i,mu_i)) / scale

        INPUTS:
           Y     -- response variable
           mu    -- mean parameter
           scale -- optional scale in denominator of deviance

        OUTPUTS: dev
           dev   -- DEV, as described aboce

        """

        return np.power(self.devresid(Y, mu), 2).sum() / scale

    def devresid(self, Y, mu):
        """
        The deviance residuals, defined as the residuals
        in the deviance.

        Without knowing the link, they default to Pearson residuals

        resid_P = (Y - mu) * sqrt(weight(mu))

        INPUTS:
           Y     -- response variable
           mu    -- mean parameter

        OUTPUTS: resid
           resid -- deviance residuals
        """

        return (Y - mu) * np.sqrt(self.weights(mu))

    def fitted(self, eta):
        """
        Fitted values based on linear predictors eta.

        INPUTS:
           eta  -- values of linear predictors, say,
                   X beta in a generalized linear model.

        OUTPUTS: mu
           mu   -- link.inverse(eta), mean parameter based on eta

        """
        return self.link.inverse(eta)

    def predict(self, mu):
        """
        Linear predictors based on given mu values.

        INPUTS:
           mu   -- mean parameter of one-parameter exponential family

        OUTPUTS: eta
           eta  -- link(mu), linear predictors, based on
                   mean parameters mu

        """
        return self.link(mu)

class Poisson(Family):

    """
    Poisson exponential family.

    INPUTS:
       link      -- a Link instance

    """

    links = [L.log, L.identity, L.sqrt]
    variance = V.mu
    valid = [0, np.inf]

    def __init__(self, link=L.log):
        self.variance = Poisson.variance
        self.link = link

    def devresid(self, Y, mu):
        """
        Poisson deviance residual

        INPUTS:
           Y     -- response variable
           mu    -- mean parameter

        OUTPUTS: resid
           resid -- deviance residuals

        """
        return np.sign(Y - mu) * np.sqrt(2 * Y * np.log(Y / mu) - 2 * (Y - mu))

class Gaussian(Family):

    """
    Gaussian exponential family.

    INPUTS:
       link      -- a Link instance

    """

    links = [L.log, L.identity, L.inverse]
    variance = V.constant

    def __init__(self, link=L.identity):
        self.variance = Gaussian.variance
        self.link = link

    def devresid(self, Y, mu, scale=1.):
        """
        Gaussian deviance residual

        INPUTS:
           Y     -- response variable
           mu    -- mean parameter
           scale -- optional scale in denominator (after taking sqrt)

        OUTPUTS: resid
           resid -- deviance residuals
        """

        return (Y - mu) / np.sqrt(self.variance(mu) * scale)

class Gamma(Family):

    """
    Gamma exponential family.

    INPUTS:
       link      -- a Link instance

    BUGS:
       no deviance residuals?

    """

    links = [L.log, L.identity, L.inverse]
    variance = V.mu_squared

    def __init__(self, link=L.identity):
        self.variance = Gamma.variance
        self.link = link


class Binomial(Family):

    """
    Binomial exponential family.

    INPUTS:
       link      -- a Link instance
       n         -- number of trials for Binomial
    """

    links = [L.logit, L.probit, L.cauchy, L.log, L.cloglog]
    variance = V.binary

    def __init__(self, link=L.logit, n=1):
        self.n = n
        self.variance = V.Binomial(n=self.n)
        self.link = link

    def devresid(self, Y, mu):
        """
        Binomial deviance residual

        INPUTS:
           Y     -- response variable
           mu    -- mean parameter

        OUTPUTS: resid
           resid -- deviance residuals

        """

        mu = self.link.clean(mu)
        return np.sign(Y - mu) * np.sqrt(-2 * (Y * np.log(mu / self.n) + (self.n - Y) * np.log(1 - mu / self.n)))

class InverseGaussian(Family):

    """
    InverseGaussian exponential family.

    INPUTS:
       link      -- a Link instance
       n         -- number of trials for Binomial

    """

    links = [L.inverse_squared, L.inverse, L.identity, L.log]
    variance = V.mu_cubed

    def __init__(self, link=L.identity):
        self.n = n
        self.variance = InverseGaussian.variance
        self.link = link



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np

from nipy.algorithms.statistics.empirical_pvalue import NormalEmpiricalNull

x = np.c_[np.random.normal(size=1e4),
          np.random.normal(scale=4, size=1e4)]

enn = NormalEmpiricalNull(x)
enn.threshold(verbose=True)



""" Testing compat3 module
"""
from __future__ import with_statement

from nibabel.py3k import asstr, asbytes

from ..compat3 import to_str, open4csv

from nose.tools import (assert_true, assert_false, assert_raises,
                        assert_equal, assert_not_equal)

from nibabel.tmpdirs import InTemporaryDirectory


def test_to_str():
    # Test routine to convert to string
    assert_equal('1', to_str(1))
    assert_equal('1.0', to_str(1.0))
    assert_equal('from', to_str(asstr('from')))
    assert_equal('from', to_str(asbytes('from')))


def test_open4csv():
    # Test opening of csv files
    import csv
    contents = [['oh', 'my', 'G'],
                ['L', 'O', 'L'],
                ['when', 'cleaning', 'windas']]
    with InTemporaryDirectory():
        with open4csv('my.csv', 'w') as fobj:
            writer = csv.writer(fobj)
            writer.writerows(contents)
        with open4csv('my.csv', 'r') as fobj:
            dialect = csv.Sniffer().sniff(fobj.read())
            fobj.seek(0)
            reader = csv.reader(fobj, dialect)
            back = list(reader)
    assert_equal(contents, back)
    assert_raises(ValueError, open4csv, 'my.csv', 'rb')
    assert_raises(ValueError, open4csv, 'my.csv', 'wt')



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
This plot demonstrates a neuronal model that is a sum
of delta functions times coefficient values
"""

import matplotlib.pyplot as plt

# Coefficients for a and b
ba = 1
bb = -2

# Times for a and b
ta = [0,4,8,12,16]
tb = [2,6,10,14,18]

for t in ta:
    plt.plot([t,t],[0,ba],c='r')
for t in tb:
    plt.plot([t,t],[0,bb],c='b')

a = plt.gca()
a.set_xlabel(r'$t$')
a.set_ylabel(r'$n(t)$')
